# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

__all__ = [
    "ADD_ALIASES_GQL",
    "ADD_ARTIFACT_COLLECTION_TAGS_GQL",
    "ARTIFACT_BY_ID_GQL",
    "ARTIFACT_BY_NAME_GQL",
    "ARTIFACT_COLLECTION_ALIASES_GQL",
    "ARTIFACT_CREATED_BY_GQL",
    "ARTIFACT_MEMBERSHIP_BY_NAME_GQL",
    "ARTIFACT_TYPE_GQL",
    "ARTIFACT_USED_BY_GQL",
    "CREATE_REGISTRY_MEMBERS_GQL",
    "DELETE_ALIASES_GQL",
    "DELETE_ARTIFACT_COLLECTION_TAGS_GQL",
    "DELETE_ARTIFACT_GQL",
    "DELETE_ARTIFACT_PORTFOLIO_GQL",
    "DELETE_ARTIFACT_SEQUENCE_GQL",
    "DELETE_REGISTRY_GQL",
    "DELETE_REGISTRY_MEMBERS_GQL",
    "FETCH_ARTIFACT_MANIFEST_GQL",
    "FETCH_LINKED_ARTIFACTS_GQL",
    "FETCH_ORG_INFO_FROM_ENTITY_GQL",
    "FETCH_REGISTRIES_GQL",
    "FETCH_REGISTRY_GQL",
    "GET_ARTIFACT_FILES_GQL",
    "GET_ARTIFACT_FILE_URLS_GQL",
    "GET_ARTIFACT_MEMBERSHIP_FILES_GQL",
    "GET_ARTIFACT_MEMBERSHIP_FILE_URLS_GQL",
    "LINK_ARTIFACT_GQL",
    "PROJECT_ARTIFACTS_GQL",
    "PROJECT_ARTIFACT_COLLECTIONS_GQL",
    "PROJECT_ARTIFACT_COLLECTION_GQL",
    "PROJECT_ARTIFACT_TYPES_GQL",
    "PROJECT_ARTIFACT_TYPE_GQL",
    "REGISTRY_COLLECTIONS_GQL",
    "REGISTRY_TEAM_MEMBERS_GQL",
    "REGISTRY_USER_MEMBERS_GQL",
    "REGISTRY_VERSIONS_GQL",
    "RENAME_REGISTRY_GQL",
    "RUN_INPUT_ARTIFACTS_GQL",
    "RUN_OUTPUT_ARTIFACTS_GQL",
    "TYPE_INFO_GQL",
    "UNLINK_ARTIFACT_GQL",
    "UPDATE_ARTIFACT_GQL",
    "UPDATE_ARTIFACT_PORTFOLIO_GQL",
    "UPDATE_ARTIFACT_SEQUENCE_GQL",
    "UPDATE_ARTIFACT_SEQUENCE_TYPE_GQL",
    "UPDATE_TEAM_REGISTRY_ROLE_GQL",
    "UPDATE_USER_REGISTRY_ROLE_GQL",
    "UPSERT_REGISTRY_GQL",
]

DELETE_ARTIFACT_SEQUENCE_GQL = """
mutation DeleteArtifactSequence($id: ID!) {
  result: deleteArtifactSequence(input: {artifactSequenceID: $id}) {
    artifactCollection {
      __typename
      state
    }
  }
}
"""

DELETE_ARTIFACT_PORTFOLIO_GQL = """
mutation DeleteArtifactPortfolio($id: ID!) {
  result: deleteArtifactPortfolio(input: {artifactPortfolioID: $id}) {
    artifactCollection {
      __typename
      state
    }
  }
}
"""

UPDATE_ARTIFACT_SEQUENCE_GQL = """
mutation UpdateArtifactSequence($input: UpdateArtifactSequenceInput!) {
  result: updateArtifactSequence(input: $input) {
    artifactCollection {
      __typename
      ...ArtifactCollectionFragment
    }
  }
}

fragment ArtifactCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

UPDATE_ARTIFACT_PORTFOLIO_GQL = """
mutation UpdateArtifactPortfolio($input: UpdateArtifactPortfolioInput!) {
  result: updateArtifactPortfolio(input: $input) {
    artifactCollection {
      __typename
      ...ArtifactCollectionFragment
    }
  }
}

fragment ArtifactCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

UPDATE_ARTIFACT_SEQUENCE_TYPE_GQL = """
mutation UpdateArtifactSequenceType($input: MoveArtifactSequenceInput!) {
  result: moveArtifactSequence(input: $input) {
    artifactCollection {
      __typename
      ...ArtifactCollectionFragment
    }
  }
}

fragment ArtifactCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

ADD_ARTIFACT_COLLECTION_TAGS_GQL = """
mutation AddArtifactCollectionTags($input: CreateArtifactCollectionTagAssignmentsInput!) {
  result: createArtifactCollectionTagAssignments(input: $input) {
    tags {
      ...TagFragment
    }
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

DELETE_ARTIFACT_COLLECTION_TAGS_GQL = """
mutation DeleteArtifactCollectionTags($input: DeleteArtifactCollectionTagAssignmentsInput!) {
  result: deleteArtifactCollectionTagAssignments(input: $input) {
    success
  }
}
"""

PROJECT_ARTIFACT_COLLECTIONS_GQL = """
query ProjectArtifactCollections($entity: String!, $project: String!, $type: String!, $cursor: String, $perPage: Int) {
  project(entityName: $entity, name: $project) {
    artifactType(name: $type) {
      artifactCollections(after: $cursor, first: $perPage) {
        totalCount
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            __typename
            ...ArtifactCollectionFragment
          }
        }
      }
    }
  }
}

fragment ArtifactCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

PROJECT_ARTIFACT_COLLECTION_GQL = """
query ProjectArtifactCollection($entity: String!, $project: String!, $type: String!, $name: String!) {
  project(entityName: $entity, name: $project) {
    artifactType(name: $type) {
      artifactCollection(name: $name) {
        __typename
        ...ArtifactCollectionFragment
      }
    }
  }
}

fragment ArtifactCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

ARTIFACT_COLLECTION_ALIASES_GQL = """
query ArtifactCollectionAliases($id: ID!, $cursor: String, $perPage: Int = 1000) {
  artifactCollection(id: $id) {
    __typename
    aliases(after: $cursor, first: $perPage) {
      pageInfo {
        ...PageInfoFragment
      }
      edges {
        node {
          ...ArtifactAliasFragment
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

GET_ARTIFACT_FILES_GQL = """
query GetArtifactFiles($entity: String!, $project: String!, $type: String!, $name: String!, $fileNames: [String!], $cursor: String, $perPage: Int = 50) {
  project(name: $project, entityName: $entity) {
    artifactType(name: $type) {
      artifact(name: $name) {
        files(names: $fileNames, after: $cursor, first: $perPage) {
          totalCount @include(if: true)
          pageInfo {
            ...PageInfoFragment
          }
          edges {
            node {
              ...FileFragment
            }
          }
        }
      }
    }
  }
}

fragment FileFragment on File {
  __typename
  id
  name: displayName
  url
  sizeBytes
  storagePath @include(if: true)
  mimetype
  updatedAt
  digest
  md5
  directUrl
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

GET_ARTIFACT_MEMBERSHIP_FILES_GQL = """
query GetArtifactMembershipFiles($entity: String!, $project: String!, $collection: String!, $alias: String!, $fileNames: [String!], $cursor: String, $perPage: Int = 50) {
  project(name: $project, entityName: $entity) {
    artifactCollection(name: $collection) {
      __typename
      artifactMembership(aliasName: $alias) {
        files(names: $fileNames, after: $cursor, first: $perPage) {
          totalCount @include(if: true)
          pageInfo {
            ...PageInfoFragment
          }
          edges {
            node {
              ...FileFragment
            }
          }
        }
      }
    }
  }
}

fragment FileFragment on File {
  __typename
  id
  name: displayName
  url
  sizeBytes
  storagePath @include(if: true)
  mimetype
  updatedAt
  digest
  md5
  directUrl
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

GET_ARTIFACT_FILE_URLS_GQL = """
query GetArtifactFileUrls($id: ID!, $cursor: String, $perPage: Int) {
  artifact(id: $id) {
    files(after: $cursor, first: $perPage) {
      pageInfo {
        ...PageInfoFragment
      }
      edges {
        node {
          ...FileWithUrlFragment
        }
      }
    }
  }
}

fragment FileWithUrlFragment on File {
  __typename
  name
  directUrl
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

GET_ARTIFACT_MEMBERSHIP_FILE_URLS_GQL = """
query GetArtifactMembershipFileUrls($entity: String!, $project: String!, $collection: String!, $alias: String!, $cursor: String, $perPage: Int) {
  project(name: $project, entityName: $entity) {
    artifactCollection(name: $collection) {
      __typename
      artifactMembership(aliasName: $alias) {
        files(after: $cursor, first: $perPage) {
          pageInfo {
            ...PageInfoFragment
          }
          edges {
            node {
              ...FileWithUrlFragment
            }
          }
        }
      }
    }
  }
}

fragment FileWithUrlFragment on File {
  __typename
  name
  directUrl
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

PROJECT_ARTIFACT_TYPES_GQL = """
query ProjectArtifactTypes($entity: String!, $project: String!, $cursor: String, $perPage: Int) {
  project(name: $project, entityName: $entity) {
    artifactTypes(after: $cursor, first: $perPage) {
      edges {
        node {
          ...ArtifactTypeFragment
        }
      }
      pageInfo {
        ...PageInfoFragment
      }
    }
  }
}

fragment ArtifactTypeFragment on ArtifactType {
  __typename
  id
  name
  description
  createdAt
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}
"""

PROJECT_ARTIFACT_TYPE_GQL = """
query ProjectArtifactType($entity: String!, $project: String!, $type: String!) {
  project(entityName: $entity, name: $project) {
    artifactType(name: $type) {
      ...ArtifactTypeFragment
    }
  }
}

fragment ArtifactTypeFragment on ArtifactType {
  __typename
  id
  name
  description
  createdAt
}
"""

PROJECT_ARTIFACTS_GQL = """
query ProjectArtifacts($entity: String!, $project: String!, $type: String!, $collection: String!, $cursor: String, $perPage: Int = 50, $order: String, $filters: JSONString, $includeAliases: Boolean = true) {
  project(entityName: $entity, name: $project) {
    artifactType(name: $type) {
      artifactCollection(name: $collection) {
        __typename
        artifacts(after: $cursor, first: $perPage, order: $order, filters: $filters) {
          totalCount
          pageInfo {
            ...PageInfoFragment
          }
          edges {
            version
            node {
              ...ArtifactFragment
            }
          }
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

RUN_OUTPUT_ARTIFACTS_GQL = """
query RunOutputArtifacts($entity: String!, $project: String!, $run: String!, $cursor: String, $perPage: Int, $includeAliases: Boolean = true) {
  project(entityName: $entity, name: $project) {
    run(name: $run) {
      artifacts: outputArtifacts(after: $cursor, first: $perPage) {
        totalCount
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            ...ArtifactFragment
          }
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

RUN_INPUT_ARTIFACTS_GQL = """
query RunInputArtifacts($entity: String!, $project: String!, $run: String!, $cursor: String, $perPage: Int, $includeAliases: Boolean = true) {
  project(entityName: $entity, name: $project) {
    run(name: $run) {
      artifacts: inputArtifacts(after: $cursor, first: $perPage) {
        totalCount
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            ...ArtifactFragment
          }
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

FETCH_LINKED_ARTIFACTS_GQL = """
query FetchLinkedArtifacts($artifactID: ID!) {
  artifact(id: $artifactID) {
    artifactMemberships {
      edges {
        node {
          versionIndex
          aliases {
            ...ArtifactAliasFragment
          }
          artifactCollection {
            __typename
            ...CollectionInfoFragment
          }
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}
"""

FETCH_ARTIFACT_MANIFEST_GQL = """
query FetchArtifactManifest($id: ID!) {
  artifact(id: $id) {
    currentManifest {
      ...DeferredManifestFragment
    }
  }
}

fragment DeferredManifestFragment on ArtifactManifest {
  file {
    directUrl
  }
}
"""

ARTIFACT_BY_ID_GQL = """
query ArtifactByID($id: ID!, $includeAliases: Boolean = true) {
  artifact(id: $id) {
    ...ArtifactFragment
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

ARTIFACT_BY_NAME_GQL = """
query ArtifactByName($entity: String!, $project: String!, $name: String!, $enableTracking: Boolean, $includeAliases: Boolean = true) {
  project(name: $project, entityName: $entity) {
    artifact(name: $name, enableTracking: $enableTracking) {
      ...ArtifactFragment
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

ARTIFACT_MEMBERSHIP_BY_NAME_GQL = """
query ArtifactMembershipByName($entity: String!, $project: String!, $name: String!, $includeAliases: Boolean = true) {
  project(name: $project, entityName: $entity) {
    artifactCollectionMembership(name: $name) {
      ...ArtifactMembershipFragment
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment ArtifactMembershipFragment on ArtifactCollectionMembership {
  __typename
  id
  versionIndex
  aliases {
    ...ArtifactAliasFragment
  }
  artifactCollection {
    ...CollectionInfoFragment
  }
  artifact {
    ...ArtifactFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

ARTIFACT_USED_BY_GQL = """
query ArtifactUsedBy($id: ID!) {
  artifact(id: $id) {
    usedBy {
      edges {
        node {
          ...RunInfoFragment
        }
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment RunInfoFragment on Run {
  __typename
  id
  name
  project {
    ...ProjectInfoFragment
  }
}
"""

ARTIFACT_CREATED_BY_GQL = """
query ArtifactCreatedBy($id: ID!) {
  artifact(id: $id) {
    createdBy {
      __typename
      ... on Run {
        ...RunInfoFragment
      }
    }
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment RunInfoFragment on Run {
  __typename
  id
  name
  project {
    ...ProjectInfoFragment
  }
}
"""

ARTIFACT_TYPE_GQL = """
query ArtifactType($entity: String, $project: String, $name: String!) {
  project(name: $project, entityName: $entity) {
    artifact(name: $name) {
      artifactType {
        name
      }
    }
  }
}
"""

ADD_ALIASES_GQL = """
mutation AddAliases($input: AddAliasesInput!) {
  result: addAliases(input: $input) {
    success
  }
}
"""

DELETE_ALIASES_GQL = """
mutation DeleteAliases($input: DeleteAliasesInput!) {
  result: deleteAliases(input: $input) {
    success
  }
}
"""

UPDATE_ARTIFACT_GQL = """
mutation UpdateArtifact($input: UpdateArtifactInput!, $includeAliases: Boolean = true) {
  result: updateArtifact(input: $input) {
    artifact {
      ...ArtifactFragment
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

DELETE_ARTIFACT_GQL = """
mutation DeleteArtifact($input: DeleteArtifactInput!) {
  result: deleteArtifact(input: $input) {
    artifact {
      id
    }
  }
}
"""

LINK_ARTIFACT_GQL = """
mutation LinkArtifact($input: LinkArtifactInput!, $includeAliases: Boolean = true) {
  result: linkArtifact(input: $input) {
    versionIndex
    artifactMembership @include(if: true) {
      ...ArtifactMembershipFragment
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment ArtifactMembershipFragment on ArtifactCollectionMembership {
  __typename
  id
  versionIndex
  aliases {
    ...ArtifactAliasFragment
  }
  artifactCollection {
    ...CollectionInfoFragment
  }
  artifact {
    ...ArtifactFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

UNLINK_ARTIFACT_GQL = """
mutation UnlinkArtifact($input: UnlinkArtifactInput!) {
  result: unlinkArtifact(input: $input) {
    success
  }
}
"""

TYPE_INFO_GQL = """
query TypeInfo($name: String!) {
  __type(name: $name) {
    ...TypeInfoFragment
  }
}

fragment TypeInfoFragment on __Type {
  name
  fields {
    name
    args {
      name
    }
  }
  inputFields {
    name
  }
}
"""

FETCH_ORG_INFO_FROM_ENTITY_GQL = """
query FetchOrgInfoFromEntity($entity: String!) {
  entity(name: $entity) {
    organization {
      ...OrgInfoFragment
    }
    user {
      organizations {
        ...OrgInfoFragment
      }
    }
  }
}

fragment OrgInfoFragment on Organization {
  name
  orgEntity @include(if: true) {
    name
  }
}
"""

REGISTRY_VERSIONS_GQL = """
query RegistryVersions($organization: String!, $registryFilter: JSONString, $collectionFilter: JSONString, $artifactFilter: JSONString, $cursor: String, $perPage: Int, $includeAliases: Boolean = false) {
  organization(name: $organization) {
    orgEntity {
      name
      artifactMemberships(
        projectFilters: $registryFilter
        collectionFilters: $collectionFilter
        filters: $artifactFilter
        after: $cursor
        first: $perPage
      ) {
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            ...ArtifactMembershipFragment
          }
        }
      }
    }
  }
}

fragment ArtifactAliasFragment on ArtifactAlias {
  __typename
  id
  alias
}

fragment ArtifactFragment on Artifact {
  __typename
  id
  artifactSequence {
    ...SourceCollectionInfoFragment
  }
  versionIndex
  artifactType {
    name
  }
  description
  metadata
  ttlDurationSeconds @include(if: true)
  ttlIsInherited @include(if: true)
  tags @include(if: true) {
    ...TagFragment
  }
  historyStep @include(if: true)
  state
  size
  digest
  commitHash
  fileCount
  createdAt
  updatedAt
  aliases @include(if: $includeAliases) {
    artifactCollection {
      ...CollectionInfoFragment
    }
    ...ArtifactAliasFragment
  }
}

fragment ArtifactMembershipFragment on ArtifactCollectionMembership {
  __typename
  id
  versionIndex
  aliases {
    ...ArtifactAliasFragment
  }
  artifactCollection {
    ...CollectionInfoFragment
  }
  artifact {
    ...ArtifactFragment
  }
}

fragment CollectionInfoFragment on ArtifactCollection {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment SourceCollectionInfoFragment on ArtifactSequence {
  __typename
  name
  project {
    ...ProjectInfoFragment
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

REGISTRY_COLLECTIONS_GQL = """
query RegistryCollections($organization: String!, $registryFilter: JSONString, $collectionFilter: JSONString, $collectionTypes: [ArtifactCollectionType!] = [PORTFOLIO], $cursor: String, $perPage: Int) {
  organization(name: $organization) {
    orgEntity {
      name
      artifactCollections(
        projectFilters: $registryFilter
        filters: $collectionFilter
        collectionTypes: $collectionTypes
        after: $cursor
        first: $perPage
      ) {
        totalCount
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            __typename
            ...RegistryCollectionFragment
          }
        }
      }
    }
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment ProjectInfoFragment on Project {
  name
  entity {
    name
  }
}

fragment RegistryCollectionFragment on ArtifactCollection {
  __typename
  id
  name
  description
  createdAt
  project {
    ...ProjectInfoFragment
  }
  type: defaultArtifactType {
    name
  }
  tags {
    edges {
      node {
        ...TagFragment
      }
    }
  }
}

fragment TagFragment on Tag {
  __typename
  id
  name
}
"""

FETCH_REGISTRY_GQL = """
query FetchRegistry($name: String, $entity: String) {
  entity(name: $entity) {
    project(name: $name) {
      ...RegistryFragment
    }
  }
}

fragment RegistryFragment on Project {
  __typename
  id
  name
  entity {
    name
    organization {
      name
    }
  }
  description
  createdAt
  updatedAt
  access
  allowAllArtifactTypes: allowAllArtifactTypesInRegistry
  artifactTypes(includeAll: true) {
    edges {
      node {
        name
      }
    }
  }
}
"""

FETCH_REGISTRIES_GQL = """
query FetchRegistries($organization: String!, $filters: JSONString, $cursor: String, $perPage: Int) {
  organization(name: $organization) {
    orgEntity {
      projects(filters: $filters, after: $cursor, first: $perPage) {
        pageInfo {
          ...PageInfoFragment
        }
        edges {
          node {
            ...RegistryFragment
          }
        }
      }
    }
  }
}

fragment PageInfoFragment on PageInfo {
  __typename
  endCursor
  hasNextPage
}

fragment RegistryFragment on Project {
  __typename
  id
  name
  entity {
    name
    organization {
      name
    }
  }
  description
  createdAt
  updatedAt
  access
  allowAllArtifactTypes: allowAllArtifactTypesInRegistry
  artifactTypes(includeAll: true) {
    edges {
      node {
        name
      }
    }
  }
}
"""

RENAME_REGISTRY_GQL = """
mutation RenameRegistry($input: RenameProjectInput!) {
  renameProject(input: $input) {
    inserted
    project {
      ...RegistryFragment
    }
  }
}

fragment RegistryFragment on Project {
  __typename
  id
  name
  entity {
    name
    organization {
      name
    }
  }
  description
  createdAt
  updatedAt
  access
  allowAllArtifactTypes: allowAllArtifactTypesInRegistry
  artifactTypes(includeAll: true) {
    edges {
      node {
        name
      }
    }
  }
}
"""

UPSERT_REGISTRY_GQL = """
mutation UpsertRegistry($input: UpsertModelInput!) {
  upsertModel(input: $input) {
    inserted
    project {
      ...RegistryFragment
    }
  }
}

fragment RegistryFragment on Project {
  __typename
  id
  name
  entity {
    name
    organization {
      name
    }
  }
  description
  createdAt
  updatedAt
  access
  allowAllArtifactTypes: allowAllArtifactTypesInRegistry
  artifactTypes(includeAll: true) {
    edges {
      node {
        name
      }
    }
  }
}
"""

DELETE_REGISTRY_GQL = """
mutation DeleteRegistry($id: String!) {
  deleteModel(input: {id: $id}) {
    success
  }
}
"""

REGISTRY_USER_MEMBERS_GQL = """
query RegistryUserMembers($project: String!, $entity: String!) {
  project(name: $project, entityName: $entity) {
    members {
      ...UserRegistryMemberFragment
    }
  }
}

fragment RegistryRoleFragment on Role {
  name
}

fragment UserRegistryMemberFragment on ProjectMember {
  id
  name
  username
  email
  role {
    ...RegistryRoleFragment
  }
}
"""

REGISTRY_TEAM_MEMBERS_GQL = """
query RegistryTeamMembers($project: String!, $entity: String!) {
  project(name: $project, entityName: $entity) {
    teamMembers {
      ...TeamRegistryMemberFragment
    }
  }
}

fragment RegistryRoleFragment on Role {
  name
}

fragment TeamFragment on Entity {
  __typename
  id
  name
  available
  photoUrl
  readOnly
  readOnlyAdmin
  isTeam
  privateOnly
  storageBytes
  codeSavingEnabled
  defaultAccess
  isPaid
  members {
    ...TeamMemberFragment
  }
}

fragment TeamMemberFragment on Member {
  __typename
  id
  role
  pending
  email
  username
  name
  photoUrl
  accountType
  apiKey
}

fragment TeamRegistryMemberFragment on ProjectTeamMember {
  team {
    ...TeamFragment
  }
  role {
    ...RegistryRoleFragment
  }
}
"""

CREATE_REGISTRY_MEMBERS_GQL = """
mutation CreateRegistryMembers($input: CreateProjectMembersInput!) {
  result: createProjectMembers(input: $input) {
    success
  }
}
"""

DELETE_REGISTRY_MEMBERS_GQL = """
mutation DeleteRegistryMembers($input: DeleteProjectMembersInput!) {
  result: deleteProjectMembers(input: $input) {
    success
  }
}
"""

UPDATE_USER_REGISTRY_ROLE_GQL = """
mutation UpdateUserRegistryRole($input: UpdateProjectMemberInput!) {
  result: updateProjectMember(input: $input) {
    success
  }
}
"""

UPDATE_TEAM_REGISTRY_ROLE_GQL = """
mutation UpdateTeamRegistryRole($input: UpdateProjectTeamMemberInput!) {
  result: updateProjectTeamMember(input: $input) {
    success
  }
}
"""



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from wandb._pydantic import GQLResult

from .fragments import UserRegistryMemberFragment


class RegistryUserMembers(GQLResult):
    project: Optional[RegistryUserMembersProject]


class RegistryUserMembersProject(GQLResult):
    members: List[UserRegistryMemberFragment]


RegistryUserMembers.model_rebuild()
RegistryUserMembersProject.model_rebuild()



"""Actions that are triggered by W&B Automations."""

from __future__ import annotations

from typing import Any, Literal, Optional, Union

from pydantic import BeforeValidator, Field
from typing_extensions import Annotated, Self, TypeVar, get_args

from wandb._pydantic import GQLBase, GQLId
from wandb._strutils import nameof

from ._generated import (
    AlertSeverity,
    GenericWebhookActionFields,
    GenericWebhookActionInput,
    NoOpActionFields,
    NoOpTriggeredActionInput,
    NotificationActionFields,
    NotificationActionInput,
    QueueJobActionFields,
)
from ._validators import (
    JsonEncoded,
    LenientStrEnum,
    default_if_none,
    parse_input_action,
    parse_saved_action,
    upper_if_str,
)
from .integrations import SlackIntegration, WebhookIntegration

T = TypeVar("T")


# NOTE: Name shortened for readability and defined publicly for easier access
class ActionType(LenientStrEnum):
    """The type of action triggered by an automation."""

    QUEUE_JOB = "QUEUE_JOB"  # NOTE: Deprecated for creation
    NOTIFICATION = "NOTIFICATION"
    GENERIC_WEBHOOK = "GENERIC_WEBHOOK"
    NO_OP = "NO_OP"


# ------------------------------------------------------------------------------
# Saved types: for parsing response data from saved automations


# NOTE: `QueueJobActionInput` for defining a Launch job is deprecated,
# so while we allow parsing it from previously saved Automations, we deliberately
# don't currently expose it in the API for creating automations.
class SavedLaunchJobAction(QueueJobActionFields):
    action_type: Literal[ActionType.QUEUE_JOB] = ActionType.QUEUE_JOB


# FIXME: Find a better place to put these OR a better way to handle the
#   conversion from `InputAction` -> `SavedAction`.
#
# Necessary placeholder class defs for converting:
# - `SendNotification -> SavedNotificationAction`
# - `SendWebhook -> SavedWebhookAction`
#
# The "input" types (`Send{Notification,Webhook}`) will only have an `integration_id`,
# and we don't want/need to fetch the other `{Slack,Webhook}Integration` fields if
# we can avoid it.
class _SlackIntegrationStub(GQLBase):
    typename__: Annotated[
        Literal["SlackIntegration"],
        Field(alias="__typename", frozen=True, repr=False),
    ] = "SlackIntegration"
    id: GQLId


class _WebhookIntegrationStub(GQLBase):
    typename__: Annotated[
        Literal["GenericWebhookIntegration"],
        Field(alias="__typename", frozen=True, repr=False),
    ] = "GenericWebhookIntegration"
    id: GQLId


class SavedNotificationAction(NotificationActionFields, frozen=False):
    action_type: Literal[ActionType.NOTIFICATION] = ActionType.NOTIFICATION
    integration: _SlackIntegrationStub

    title: Optional[str]
    message: Optional[str]
    severity: Optional[AlertSeverity]


class SavedWebhookAction(GenericWebhookActionFields, frozen=False):
    action_type: Literal[ActionType.GENERIC_WEBHOOK] = ActionType.GENERIC_WEBHOOK
    integration: _WebhookIntegrationStub

    # We override the type of the `requestPayload` field since the original GraphQL
    # schema (and generated class) effectively defines it as a string, when we know
    # and need to anticipate the expected structure of the JSON-serialized data.
    request_payload: Optional[JsonEncoded[dict[str, Any]]] = None  # type: ignore[assignment]


class SavedNoOpAction(NoOpActionFields, frozen=False):
    action_type: Literal[ActionType.NO_OP] = ActionType.NO_OP

    no_op: Annotated[
        bool,
        BeforeValidator(default_if_none),
        Field(repr=False, frozen=True),
    ] = True
    """Placeholder field, only needed to conform to schema requirements.

    There should never be a need to set this field explicitly, as its value is ignored.
    """


# for type annotations
SavedAction = Annotated[
    Union[
        SavedLaunchJobAction,
        SavedNotificationAction,
        SavedWebhookAction,
        SavedNoOpAction,
    ],
    BeforeValidator(parse_saved_action),
    Field(discriminator="typename__"),
]
# for runtime type checks
SavedActionTypes: tuple[type, ...] = get_args(SavedAction.__origin__)  # type: ignore[attr-defined]


# ------------------------------------------------------------------------------
# Input types: for creating or updating automations
class _BaseActionInput(GQLBase):
    action_type: Annotated[ActionType, Field(frozen=True)]
    """The kind of action to be triggered."""


class SendNotification(_BaseActionInput, NotificationActionInput):
    """Defines an automation action that sends a (Slack) notification."""

    action_type: Literal[ActionType.NOTIFICATION] = ActionType.NOTIFICATION

    integration_id: GQLId
    """The ID of the Slack integration that will be used to send the notification."""

    # Note: Validation aliases preserve continuity with the prior `wandb.alert()` API.
    title: str = ""
    """The title of the sent notification."""

    message: Annotated[str, Field(validation_alias="text")] = ""
    """The message body of the sent notification."""

    severity: Annotated[
        AlertSeverity,
        BeforeValidator(upper_if_str),  # Be helpful by ensuring uppercase strings
        Field(validation_alias="level"),
    ] = AlertSeverity.INFO
    """The severity (`INFO`, `WARN`, `ERROR`) of the sent notification."""

    @classmethod
    def from_integration(
        cls,
        integration: SlackIntegration,
        *,
        title: str = "",
        text: str = "",
        level: AlertSeverity = AlertSeverity.INFO,
    ) -> Self:
        """Define a notification action that sends to the given (Slack) integration."""
        return cls(
            integration_id=integration.id, title=title, message=text, severity=level
        )


class SendWebhook(_BaseActionInput, GenericWebhookActionInput):
    """Defines an automation action that sends a webhook request."""

    action_type: Literal[ActionType.GENERIC_WEBHOOK] = ActionType.GENERIC_WEBHOOK

    integration_id: GQLId
    """The ID of the webhook integration that will be used to send the request."""

    # overrides the generated field type to parse/serialize JSON strings
    request_payload: Optional[JsonEncoded[dict[str, Any]]] = Field(  # type: ignore[assignment]
        default=None, alias="requestPayload"
    )
    """The payload, possibly with template variables, to send in the webhook request."""

    @classmethod
    def from_integration(
        cls,
        integration: WebhookIntegration,
        *,
        payload: Optional[JsonEncoded[dict[str, Any]]] = None,
    ) -> Self:
        """Define a webhook action that sends to the given (webhook) integration."""
        return cls(integration_id=integration.id, request_payload=payload)


class DoNothing(_BaseActionInput, NoOpTriggeredActionInput, frozen=True):
    """Defines an automation action that intentionally does nothing."""

    action_type: Literal[ActionType.NO_OP] = ActionType.NO_OP

    no_op: Annotated[bool, BeforeValidator(default_if_none)] = True
    """Placeholder field which exists only to satisfy backend schema requirements.

    There should never be a need to set this field explicitly, as its value is ignored.
    """


# for type annotations
InputAction = Annotated[
    Union[
        SendNotification,
        SendWebhook,
        DoNothing,
    ],
    BeforeValidator(parse_input_action),
    Field(discriminator="action_type"),
]
# for runtime type checks
InputActionTypes: tuple[type, ...] = get_args(InputAction.__origin__)  # type: ignore[attr-defined]

__all__ = [
    "ActionType",
    *(nameof(cls) for cls in InputActionTypes),
]



import os
import pathlib
from typing import TYPE_CHECKING, Sequence, Type, Union

from wandb.sdk.lib import filesystem, runid

from . import _dtypes
from ._private import MEDIA_TMP
from .base_types.media import BatchableMedia

if TYPE_CHECKING:  # pragma: no cover
    from typing import TextIO

    from wandb.sdk.artifacts.artifact import Artifact

    from ..wandb_run import Run as LocalRun


class Html(BatchableMedia):
    """W&B class for logging HTML content to W&B."""

    _log_type = "html-file"

    def __init__(
        self,
        data: Union[str, pathlib.Path, "TextIO"],
        inject: bool = True,
        data_is_not_path: bool = False,
    ) -> None:
        """Creates a W&B HTML object.

        Args:
            data:
                A string that is a path to a file with the extension ".html",
                or a string or IO object containing literal HTML.
            inject: Add a stylesheet to the HTML object. If set
                to False the HTML will pass through unchanged.
            data_is_not_path: If set to False, the data will be
                treated as a path to a file.

        Examples:
        It can be initialized by providing a path to a file:

        ```python
        with wandb.init() as run:
            run.log({"html": wandb.Html("./index.html")})
        ```

        Alternatively, it can be initialized by providing literal HTML,
        in either a string or IO object:

        ```python
        with wandb.init() as run:
            run.log({"html": wandb.Html("<h1>Hello, world!</h1>")})
        ```
        """
        super().__init__()
        data_is_path = (
            isinstance(data, (str, pathlib.Path))
            and os.path.isfile(data)
            and os.path.splitext(data)[1] == ".html"
        ) and not data_is_not_path
        data_path = ""
        if data_is_path:
            data_path = str(data)
            with open(data_path, encoding="utf-8") as file:
                self.html = file.read()
        elif isinstance(data, str):
            self.html = data
        elif hasattr(data, "read"):
            if hasattr(data, "seek"):
                data.seek(0)
            self.html = data.read()
        else:
            raise ValueError("data must be a string or an io object")

        if inject:
            self.inject_head()

        if inject or not data_is_path:
            tmp_path = os.path.join(MEDIA_TMP.name, runid.generate_id() + ".html")
            with open(tmp_path, "w", encoding="utf-8") as out:
                out.write(self.html)

            self._set_file(tmp_path, is_tmp=True)
        else:
            self._set_file(data_path, is_tmp=False)

    def inject_head(self) -> None:
        """Inject a <head> tag into the HTML.

        <!-- lazydoc-ignore: internal -->
        """
        join = ""
        if "<head>" in self.html:
            parts = self.html.split("<head>", 1)
            parts[0] = parts[0] + "<head>"
        elif "<html>" in self.html:
            parts = self.html.split("<html>", 1)
            parts[0] = parts[0] + "<html><head>"
            parts[1] = "</head>" + parts[1]
        else:
            parts = ["", self.html]
        parts.insert(
            1,
            '<base target="_blank"><link rel="stylesheet" type="text/css" href="https://app.wandb.ai/normalize.css" />',
        )
        self.html = join.join(parts).strip()

    @classmethod
    def get_media_subdir(cls: Type["Html"]) -> str:
        """Get media subdirectory.

        "<!-- lazydoc-ignore-classmethod: internal -->
        """
        return os.path.join("media", "html")

    def to_json(self, run_or_artifact: Union["LocalRun", "Artifact"]) -> dict:
        """Returns the JSON representation expected by the backend.

        <!-- lazydoc-ignore: internal -->
        """
        json_dict = super().to_json(run_or_artifact)
        json_dict["_type"] = self._log_type
        return json_dict

    @classmethod
    def from_json(
        cls: Type["Html"], json_obj: dict, source_artifact: "Artifact"
    ) -> "Html":
        """Deserialize a JSON object into it's class representation.

        "<!-- lazydoc-ignore-classmethod: internal -->
        """
        return cls(source_artifact.get_entry(json_obj["path"]).download(), inject=False)

    @classmethod
    def seq_to_json(
        cls: Type["Html"],
        seq: Sequence["BatchableMedia"],
        run: "LocalRun",
        key: str,
        step: Union[int, str],
    ) -> dict:
        """Convert a sequence of HTML objects to a JSON representation.

        "<!-- lazydoc-ignore-classmethod: internal -->
        """
        base_path = os.path.join(run.dir, cls.get_media_subdir())
        filesystem.mkdir_exists_ok(base_path)

        meta = {
            "_type": "html",
            "count": len(seq),
            "html": [h.to_json(run) for h in seq],
        }
        return meta


class _HtmlFileType(_dtypes.Type):
    name = "html-file"
    types = [Html]


_dtypes.TypeRegistry.add(_HtmlFileType)



from __future__ import annotations

import abc
import dataclasses
import pathlib

from typing_extensions import final, override

from wandb.errors import AuthenticationError

from . import validation
from .host_url import HostUrl


# We use an abstract base class instead of a union because
#   (1) All Auth subtypes have a 'host' property and a safe repr
#   (2) Auth should be treated as an open union, meaning typecheckers should
#       not consider any list of isinstance() checks exhaustive and should
#       always require a fallback case
class Auth(abc.ABC):
    """Credentials that give access to a W&B server."""

    @abc.abstractmethod
    def __init__(self, *, host: str | HostUrl) -> None:
        if isinstance(host, str):
            host = HostUrl(host)
        self._host = host

    @property
    def host(self) -> HostUrl:
        """The W&B server for which the credentials are valid."""
        return self._host

    @final
    @override
    def __repr__(self) -> str:
        return f"<{type(self).__name__} host={self.host.url!r}>"

    @final
    @override
    def __str__(self) -> str:
        return repr(self)


@final
class AuthApiKey(Auth):
    """An API key for connecting to a W&B server."""

    @override
    def __init__(self, *, host: str | HostUrl, api_key: str) -> None:
        """Initialize AuthApiKey.

        Args:
            host: The W&B server URL.
            api_key: The API key.

        Raises:
            ValueError: If the host is invalid.
            AuthenticationError: If the API key is in an invalid format.
        """
        super().__init__(host=host)

        if problems := validation.check_api_key(api_key):
            raise AuthenticationError(problems)

        self._api_key = api_key

    @property
    def api_key(self) -> str:
        """The API key."""
        return self._api_key


@final
class AuthIdentityTokenFile(Auth):
    """A path to a file storing a JWT with OIDC credentials."""

    @override
    def __init__(self, *, host: str | HostUrl, path: str) -> None:
        super().__init__(host=host)
        self._identity_token_file = pathlib.Path(path)

    @property
    def path(self) -> pathlib.Path:
        """Path to a file storing a JWT identity token."""
        return self._identity_token_file


@dataclasses.dataclass(frozen=True)
class AuthWithSource:
    """Credentials with information about where they came from."""

    auth: Auth

    source: str
    """A file path or environment variable."""



from __future__ import annotations

import json
import logging
import os
import re
import shutil
import sys
import traceback
from base64 import b64encode
from typing import Any

import IPython
import IPython.display
import requests
from IPython.core.magic import Magics, line_cell_magic, magics_class
from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring
from requests.compat import urljoin

import wandb
import wandb.util
from wandb.sdk import wandb_setup
from wandb.sdk.lib import filesystem

logger = logging.getLogger(__name__)


def display_if_magic_is_used(run: wandb.Run) -> bool:
    """Display a run's page if the cell has the %%wandb cell magic.

    Args:
        run: The run to display.

    Returns:
        Whether the %%wandb cell magic was present.
    """
    if not _current_cell_wandb_magic:
        return False

    _current_cell_wandb_magic.display_if_allowed(run)
    return True


class _WandbCellMagicState:
    """State for a cell with the %%wandb cell magic."""

    def __init__(self, *, height: int) -> None:
        """Initializes the %%wandb cell magic state.

        Args:
            height: The desired height for displayed iframes.
        """
        self._height = height
        self._already_displayed = False

    def display_if_allowed(self, run: wandb.Run) -> None:
        """Display a run's iframe if one is not already displayed.

        Args:
            run: The run to display.
        """
        if self._already_displayed:
            return
        self._already_displayed = True

        _display_wandb_run(run, height=self._height)


_current_cell_wandb_magic: _WandbCellMagicState | None = None


def _display_by_wandb_path(path: str, *, height: int) -> None:
    """Display a wandb object (usually in an iframe) given its URI.

    Args:
        path: A path to a run, sweep, project, report, etc.
        height: Height of the iframe in pixels.
    """
    api = wandb.Api()

    try:
        obj = api.from_path(path)

        IPython.display.display_html(
            obj.to_html(height=height),
            raw=True,
        )
    except wandb.Error:
        traceback.print_exc()
        IPython.display.display_html(
            f"Path {path!r} does not refer to a W&B object you can access.",
            raw=True,
        )


def _display_wandb_run(run: wandb.Run, *, height: int) -> None:
    """Display a run (usually in an iframe).

    Args:
        run: The run to display.
        height: Height of the iframe in pixels.
    """
    IPython.display.display_html(
        run.to_html(height=height),
        raw=True,
    )


@magics_class
class WandBMagics(Magics):
    def __init__(self, shell):
        super().__init__(shell)

    @magic_arguments()
    @argument(
        "path",
        default=None,
        nargs="?",
        help="The path to a resource you want to display.",
    )
    @argument(
        "-h",
        "--height",
        default=420,
        type=int,
        help="The height of the iframe in pixels.",
    )
    @line_cell_magic
    def wandb(self, line: str, cell: str | None = None) -> None:
        """Display wandb resources in Jupyter.

        This can be used as a line magic:

            %wandb USERNAME/PROJECT/runs/RUN_ID

        Or as a cell magic:

            %%wandb -h 1024
            with wandb.init() as run:
                run.log({"loss": 1})
        """
        global _current_cell_wandb_magic

        args = parse_argstring(self.wandb, line)
        path: str | None = args.path
        height: int = args.height

        if path:
            _display_by_wandb_path(path, height=height)
            displayed = True
        elif run := wandb_setup.singleton().most_recent_active_run:
            _display_wandb_run(run, height=height)
            displayed = True
        else:
            displayed = False

        # If this is being used as a line magic ("%wandb"), we are done.
        # When used as a cell magic ("%%wandb"), we must run the cell.
        if cell is None:
            return

        if not displayed:
            _current_cell_wandb_magic = _WandbCellMagicState(height=height)

        try:
            IPython.get_ipython().run_cell(cell)
        finally:
            _current_cell_wandb_magic = None


def notebook_metadata_from_jupyter_servers_and_kernel_id():
    # When running in VS Code's notebook extension,
    # the extension creates a temporary file to start the kernel.
    # This file is not actually the same as the notebook file.
    #
    # The real notebook path is stored in the user namespace
    # under the key "__vsc_ipynb_file__"
    try:
        from IPython import get_ipython

        ipython = get_ipython()
        if ipython is not None:
            notebook_path = ipython.kernel.shell.user_ns.get("__vsc_ipynb_file__")
            if notebook_path:
                return {
                    "root": os.path.dirname(notebook_path),
                    "path": notebook_path,
                    "name": os.path.basename(notebook_path),
                }
    except ModuleNotFoundError:
        return None

    servers, kernel_id = jupyter_servers_and_kernel_id()
    for s in servers:
        if s.get("password"):
            raise ValueError("Can't query password protected kernel")
        res = requests.get(
            urljoin(s["url"], "api/sessions"), params={"token": s.get("token", "")}
        ).json()
        for nn in res:
            if isinstance(nn, dict) and nn.get("kernel") and "notebook" in nn:
                if nn["kernel"]["id"] == kernel_id:
                    return {
                        "root": s.get("root_dir", s.get("notebook_dir", os.getcwd())),
                        "path": nn["notebook"]["path"],
                        "name": nn["notebook"]["name"],
                    }

    if not kernel_id:
        return None


def notebook_metadata(silent: bool) -> dict[str, str]:
    """Attempt to query jupyter for the path and name of the notebook file.

    This can handle different jupyter environments, specifically:

    1. Colab
    2. Kaggle
    3. JupyterLab
    4. Notebooks
    5. Other?
    """
    error_message = (
        "Failed to detect the name of this notebook. You can set it manually"
        " with the WANDB_NOTEBOOK_NAME environment variable to enable code"
        " saving."
    )
    try:
        jupyter_metadata = notebook_metadata_from_jupyter_servers_and_kernel_id()

        # Colab:
        # request the most recent contents
        ipynb = attempt_colab_load_ipynb()
        if ipynb is not None and jupyter_metadata is not None:
            return {
                "root": "/content",
                "path": jupyter_metadata["path"],
                "name": jupyter_metadata["name"],
            }

        # Kaggle:
        if wandb.util._is_kaggle():
            # request the most recent contents
            ipynb = attempt_kaggle_load_ipynb()
            if ipynb:
                return {
                    "root": "/kaggle/working",
                    "path": ipynb["metadata"]["name"],
                    "name": ipynb["metadata"]["name"],
                }

        if jupyter_metadata:
            return jupyter_metadata
    except Exception:
        logger.exception(error_message)

    wandb.termerror(error_message)
    return {}


def jupyter_servers_and_kernel_id():
    """Return a list of servers and the current kernel_id.

    Used to query for the name of the notebook.
    """
    try:
        import ipykernel  # type: ignore

        kernel_id = re.search(
            "kernel-(.*).json", ipykernel.connect.get_connection_file()
        ).group(1)
        # We're either in jupyterlab or a notebook, lets prefer the newer jupyter_server package
        serverapp = wandb.util.get_module("jupyter_server.serverapp")
        notebookapp = wandb.util.get_module("notebook.notebookapp")
        servers = []
        if serverapp is not None:
            servers.extend(list(serverapp.list_running_servers()))
        if notebookapp is not None:
            servers.extend(list(notebookapp.list_running_servers()))
    except (AttributeError, ValueError, ImportError):
        return [], None

    return servers, kernel_id


def attempt_colab_load_ipynb():
    colab = wandb.util.get_module("google.colab")
    if colab:
        # This isn't thread safe, never call in a thread
        response = colab._message.blocking_request("get_ipynb", timeout_sec=5)
        if response:
            return response["ipynb"]


def attempt_kaggle_load_ipynb():
    kaggle = wandb.util.get_module("kaggle_session")
    if not kaggle:
        return None

    try:
        client = kaggle.UserSessionClient()
        parsed = json.loads(client.get_exportable_ipynb()["source"])
        # TODO: couldn't find a way to get the name of the notebook...
        parsed["metadata"]["name"] = "kaggle.ipynb"
    except Exception:
        wandb.termerror("Unable to load kaggle notebook.")
        logger.exception("Unable to load kaggle notebook.")
        return None

    return parsed


class Notebook:
    def __init__(self, settings: wandb.Settings) -> None:
        self.outputs: dict[int, Any] = {}
        self.settings = settings
        self.shell = IPython.get_ipython()

    def save_display(self, exc_count, data_with_metadata):
        self.outputs[exc_count] = self.outputs.get(exc_count, [])

        # byte values such as images need to be encoded in base64
        # otherwise nbformat.v4.new_output will throw a NotebookValidationError
        data = data_with_metadata["data"]
        b64_data = {}
        for key in data:
            val = data[key]
            if isinstance(val, bytes):
                b64_data[key] = b64encode(val).decode("utf-8")
            else:
                b64_data[key] = val

        self.outputs[exc_count].append(
            {"data": b64_data, "metadata": data_with_metadata["metadata"]}
        )

    def probe_ipynb(self):
        """Return notebook as dict or None."""
        relpath = self.settings.x_jupyter_path
        if relpath:
            if os.path.exists(relpath):
                with open(relpath) as json_file:
                    data = json.load(json_file)
                    return data

        colab_ipynb = attempt_colab_load_ipynb()
        if colab_ipynb:
            return colab_ipynb

        kaggle_ipynb = attempt_kaggle_load_ipynb()
        if kaggle_ipynb and len(kaggle_ipynb["cells"]) > 0:
            return kaggle_ipynb

        return

    def save_ipynb(self) -> bool:
        if not self.settings.save_code:
            logger.info("not saving jupyter notebook")
            return False
        ret = False
        try:
            ret = self._save_ipynb()
        except Exception:
            wandb.termerror("Failed to save notebook.")
            logger.exception("Problem saving notebook.")
        return ret

    def _save_ipynb(self) -> bool:
        relpath = self.settings.x_jupyter_path
        logger.info("looking for notebook: %s", relpath)
        if relpath:
            if os.path.exists(relpath):
                shutil.copy(
                    relpath,
                    os.path.join(
                        self.settings._tmp_code_dir, os.path.basename(relpath)
                    ),
                )
                return True

        # TODO: likely only save if the code has changed
        colab_ipynb = attempt_colab_load_ipynb()
        if colab_ipynb:
            try:
                jupyter_metadata = (
                    notebook_metadata_from_jupyter_servers_and_kernel_id()
                )
                nb_name = jupyter_metadata["name"]
            except Exception:
                nb_name = "colab.ipynb"
            if not nb_name.endswith(".ipynb"):
                nb_name += ".ipynb"
            with open(
                os.path.join(
                    self.settings._tmp_code_dir,
                    nb_name,
                ),
                "w",
                encoding="utf-8",
            ) as f:
                f.write(json.dumps(colab_ipynb))
            return True

        kaggle_ipynb = attempt_kaggle_load_ipynb()
        if kaggle_ipynb and len(kaggle_ipynb["cells"]) > 0:
            with open(
                os.path.join(
                    self.settings._tmp_code_dir, kaggle_ipynb["metadata"]["name"]
                ),
                "w",
                encoding="utf-8",
            ) as f:
                f.write(json.dumps(kaggle_ipynb))
            return True

        return False

    def save_history(self, run: wandb.Run):
        """This saves all cell executions in the current session as a new notebook."""
        try:
            from nbformat import v4, validator, write  # type: ignore
        except ImportError:
            wandb.termerror(
                "The nbformat package was not found."
                " It is required to save notebook history."
            )
            return
        # TODO: some tests didn't patch ipython properly?
        if self.shell is None:
            return
        cells = []
        hist = list(self.shell.history_manager.get_range(output=True))
        if len(hist) <= 1 or not self.settings.save_code:
            logger.info("not saving jupyter history")
            return
        try:
            for _, execution_count, exc in hist:
                if exc[1]:
                    # TODO: capture stderr?
                    outputs = [
                        v4.new_output(output_type="stream", name="stdout", text=exc[1])
                    ]
                else:
                    outputs = []
                if self.outputs.get(execution_count):
                    for out in self.outputs[execution_count]:
                        outputs.append(
                            v4.new_output(
                                output_type="display_data",
                                data=out["data"],
                                metadata=out["metadata"] or {},
                            )
                        )
                cells.append(
                    v4.new_code_cell(
                        execution_count=execution_count, source=exc[0], outputs=outputs
                    )
                )
            if hasattr(self.shell, "kernel"):
                language_info = self.shell.kernel.language_info
            else:
                language_info = {"name": "python", "version": sys.version}
            logger.info("saving %i cells to _session_history.ipynb", len(cells))
            nb = v4.new_notebook(
                cells=cells,
                metadata={
                    "kernelspec": {
                        "display_name": f"Python {sys.version_info[0]}",
                        "name": f"python{sys.version_info[0]}",
                        "language": "python",
                    },
                    "language_info": language_info,
                },
            )
            state_path = os.path.join("code", "_session_history.ipynb")
            run._set_config_wandb("session_history", state_path)
            filesystem.mkdir_exists_ok(os.path.join(self.settings.files_dir, "code"))
            with open(
                os.path.join(self.settings._tmp_code_dir, "_session_history.ipynb"),
                "w",
                encoding="utf-8",
            ) as f:
                write(nb, f, version=4)
            with open(
                os.path.join(self.settings.files_dir, state_path),
                "w",
                encoding="utf-8",
            ) as f:
                write(nb, f, version=4)
        except (OSError, validator.NotebookValidationError):
            wandb.termerror("Unable to save notebook session history.")
            logger.exception("Unable to save notebook session history.")



from __future__ import annotations

import atexit
import contextlib
import functools
import os
import pathlib
import sys
import threading
from types import TracebackType
from typing import Any, Callable, Literal, TypeVar
from urllib.parse import quote

from typing_extensions import Concatenate, Never, ParamSpec

_P = ParamSpec("_P")
_T = TypeVar("_T")

SENTRY_DEFAULT_DSN = (
    "https://2592b1968ea94cca9b5ef5e348e094a7@o151352.ingest.sentry.io/4504800232407040"
)

SessionStatus = Literal["ok", "exited", "crashed", "abnormal"]


def _guard(
    method: Callable[Concatenate[Sentry, _P], _T],
) -> Callable[Concatenate[Sentry, _P], _T | None]:
    """Make a Sentry method safe, lazy, and non-raising.

    The wrapped method becomes a no-op if Sentry is disabled,
    this instance belongs to a different PID, or lazy boot fails
    """

    @functools.wraps(method)
    def wrapper(
        self: Sentry,
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> _T | None:
        if not self._enabled:
            return None

        # If this instance belongs to a different process (fork happened),
        # do nothing; get_sentry() will create a fresh instance for the child.
        if self._pid != os.getpid():
            return None

        if not self._booted and not self._boot():
            return None

        try:
            return method(self, *args, **kwargs)
        except Exception as e:
            if method.__name__ != "exception":
                # Best-effort logging of wrapper-level failures.
                with contextlib.suppress(Exception):
                    self.exception(f"Error in {method.__name__}: {e}")
            return None

    return wrapper


class Sentry:
    def __init__(self, *, pid: int) -> None:
        from wandb import env as _env

        self._pid: int = pid
        self._enabled: bool = bool(_env.error_reporting_enabled())
        self._booted: bool = False
        self._boot_lock = threading.Lock()
        self._atexit_registered: bool = False

        self._sent_messages: set[str] = set()
        self._sdk: Any | None = None  # will hold the sentry_sdk module after boot
        self.scope: Any | None = None

        self.dsn: str | None = os.environ.get(_env.SENTRY_DSN, SENTRY_DEFAULT_DSN)

    @property
    def environment(self) -> str:
        is_git = pathlib.Path(__file__).parent.parent.parent.joinpath(".git").exists()
        return "development" if is_git else "production"

    def _boot(self) -> bool:
        """Import sentry_sdk and set up client/scope."""
        from wandb import __version__

        with self._boot_lock:
            if not self._enabled:
                return False

            if self._booted:
                return True

            try:
                import sentry_sdk  # type: ignore
                import sentry_sdk.scope  # type: ignore
                import sentry_sdk.utils  # type: ignore

                self._sdk = sentry_sdk

                client = self._sdk.Client(
                    dsn=self.dsn,
                    default_integrations=False,
                    environment=self.environment,
                    release=__version__,
                )
                scope = self._sdk.get_global_scope().fork()
                scope.clear()
                scope.set_client(client)

                self.scope = scope
                self._booted = True

                if not self._atexit_registered:
                    atexit.register(self.end_session)
                    self._atexit_registered = True

            except Exception:
                # Disable on any failure.
                self._enabled = False
                self._booted = False
                self._sdk = None
                self.scope = None

                return False

            return True

    @_guard
    def message(
        self,
        message: str,
        repeat: bool = True,
        level: str = "info",
    ) -> str | None:
        if not repeat and message in self._sent_messages:
            return None
        self._sent_messages.add(message)
        with self._sdk.scope.use_isolation_scope(self.scope):  # type: ignore
            return self._sdk.capture_message(message, level=level)  # type: ignore

    @_guard
    def exception(
        self,
        exc: str
        | BaseException
        | tuple[
            type[BaseException] | None,
            BaseException | None,
            TracebackType | None,
        ]
        | None,
        handled: bool = False,
        status: SessionStatus | None = None,
    ) -> str | None:
        if isinstance(exc, str):
            exc_info = self._sdk.utils.exc_info_from_error(Exception(exc))  # type: ignore
        elif isinstance(exc, BaseException):
            exc_info = self._sdk.utils.exc_info_from_error(exc)  # type: ignore
        else:
            exc_info = sys.exc_info()

        event, _ = self._sdk.utils.event_from_exception(  # type: ignore
            exc_info,
            client_options=self.scope.get_client().options,  # type: ignore
            mechanism={"type": "generic", "handled": handled},
        )
        event_id = None
        with contextlib.suppress(Exception):
            with self._sdk.scope.use_isolation_scope(self.scope):  # type: ignore
                event_id = self._sdk.capture_event(event)  # type: ignore

        status = status or ("crashed" if not handled else "errored")  # type: ignore
        self.mark_session(status=status)

        client = self.scope.get_client()  # type: ignore
        if client is not None:
            client.flush()
        return event_id

    def reraise(self, exc: Any) -> Never:
        """Re-raise after logging, preserving traceback. Safe if disabled."""
        try:
            self.exception(exc)  # @_guard applies here
        finally:
            _, _, tb = sys.exc_info()
            if tb is not None and hasattr(exc, "with_traceback"):
                raise exc.with_traceback(tb)
            raise exc

    @_guard
    def start_session(self) -> None:
        if self.scope is None:
            return
        if self.scope._session is None:
            self.scope.start_session()

    @_guard
    def end_session(self) -> None:
        if self.scope is None:
            return
        client = self.scope.get_client()
        session = self.scope._session
        if session is not None and client is not None:
            self.scope.end_session()
            client.flush()

    @_guard
    def mark_session(self, status: SessionStatus | None = None) -> None:
        if self.scope is None:
            return
        session = self.scope._session
        if session is not None:
            session.update(status=status)

    @_guard
    def configure_scope(
        self,
        tags: dict[str, Any] | None = None,
        process_context: str | None = None,
    ) -> None:
        import wandb.util

        if self.scope is None:
            return

        settings_tags = (
            "entity",
            "project",
            "run_id",
            "run_url",
            "sweep_url",
            "sweep_id",
            "deployment",
            "launch",
            "_platform",
        )

        if process_context:
            self.scope.set_tag("process_context", process_context)

        if tags is None:
            return None

        for tag in settings_tags:
            val = tags.get(tag, None)
            if val not in (None, ""):
                self.scope.set_tag(tag, val)

        if tags.get("_colab", None):
            python_runtime = "colab"
        elif tags.get("_jupyter", None):
            python_runtime = "jupyter"
        elif tags.get("_ipython", None):
            python_runtime = "ipython"
        else:
            python_runtime = "python"
        self.scope.set_tag("python_runtime", python_runtime)

        # Construct run_url and sweep_url given run_id and sweep_id.
        for obj in ("run", "sweep"):
            obj_id, obj_url = f"{obj}_id", f"{obj}_url"
            if tags.get(obj_url, None):
                continue
            try:
                app_url = wandb.util.app_url(tags["base_url"])  # type: ignore[index]
                entity, project = (quote(tags[k]) for k in ("entity", "project"))  # type: ignore[index]
                self.scope.set_tag(
                    obj_url,
                    f"{app_url}/{entity}/{project}/{obj}s/{tags[obj_id]}",
                )
            except Exception:
                pass

        email = tags.get("email")
        if email:
            self.scope.user = {"email": email}

        self.start_session()


_singleton: Sentry | None = None
_singleton_lock = threading.Lock()


def get_sentry() -> Sentry:
    """Return the Sentry singleton for the current process (fork-aware).

    Creates a new instance in child processes after fork.
    Thread-safe within each process.
    """
    global _singleton

    pid = os.getpid()

    with _singleton_lock:
        if _singleton is not None and _singleton._pid == pid:
            return _singleton

        if _singleton is None or _singleton._pid != pid:
            _singleton = Sentry(pid=pid)

        return _singleton



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from wandb._pydantic import GQLResult

from .fragments import TagFragment


class AddArtifactCollectionTags(GQLResult):
    result: Optional[AddArtifactCollectionTagsResult]


class AddArtifactCollectionTagsResult(GQLResult):
    tags: List[TagFragment]


AddArtifactCollectionTags.model_rebuild()
AddArtifactCollectionTagsResult.model_rebuild()



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLId, GQLResult


class DeleteArtifact(GQLResult):
    result: Optional[DeleteArtifactResult]


class DeleteArtifactResult(GQLResult):
    artifact: DeleteArtifactResultArtifact


class DeleteArtifactResultArtifact(GQLResult):
    id: GQLId


DeleteArtifact.model_rebuild()
DeleteArtifactResult.model_rebuild()



"""Support for sklearn datatypes.

May raise MissingDependencyError on import.
"""

from __future__ import annotations

import pickle

from typing_extensions import Any, TypeIs

import wandb

from . import errors

try:
    from sklearn.base import BaseEstimator
except ImportError as e:
    warning = (
        "`sklearn` not installed >>"
        " @wandb_log(models=True) may not auto log your model!"
    )
    raise errors.MissingDependencyError(warning=warning) from e


def is_estimator(data: Any) -> TypeIs[BaseEstimator]:
    """Returns whether the data is an sklearn BaseEstimator."""
    return isinstance(data, BaseEstimator)


def use_estimator(
    name: str,
    run: wandb.Run | None,
    testing: bool = False,
) -> str | None:
    """Log a dependency on an sklearn estimator.

    Args:
        name: Name of the input.
        run: The run to update.
        testing: True in unit tests.
    """
    if testing:
        return "models"
    assert run

    wandb.termlog(f"Using artifact: {name} (sklearn BaseEstimator)")
    run.use_artifact(f"{name}:latest")
    return None


def track_estimator(
    name: str,
    data: BaseEstimator,
    run: wandb.Run | None,
    testing: bool = False,
) -> str | None:
    """Log an sklearn estimator output as an artifact.

    Args:
        name: The output's name.
        data: The output's value.
        run: The run to update.
        testing: True in unit tests.
    """
    if testing:
        return "BaseEstimator"
    assert run

    artifact = wandb.Artifact(name, type="model")
    with artifact.new_file(f"{name}.pkl", "wb") as f:
        pickle.dump(data, f)

    wandb.termlog(f"Logging artifact: {name} (sklearn BaseEstimator)")
    run.log_artifact(artifact)
    return None



"""WandB storage policy."""

from __future__ import annotations

import concurrent.futures
import hashlib
import logging
import os
import shutil
from collections import deque
from operator import itemgetter
from typing import TYPE_CHECKING, Any
from urllib.parse import quote

from typing_extensions import assert_never

from wandb.errors.term import termwarn
from wandb.proto import wandb_internal_pb2 as pb
from wandb.sdk.artifacts._gqlutils import server_supports
from wandb.sdk.artifacts._models.storage import StoragePolicyConfig
from wandb.sdk.artifacts.artifact_file_cache import (
    ArtifactFileCache,
    get_artifact_file_cache,
)
from wandb.sdk.artifacts.staging import get_staging_dir
from wandb.sdk.artifacts.storage_handlers.multi_handler import MultiHandler
from wandb.sdk.artifacts.storage_handlers.tracking_handler import TrackingHandler
from wandb.sdk.artifacts.storage_layout import StorageLayout
from wandb.sdk.artifacts.storage_policies._multipart import (
    MAX_MULTI_UPLOAD_SIZE,
    MIN_MULTI_UPLOAD_SIZE,
    KiB,
    calc_part_size,
    multipart_download,
    scan_chunks,
)
from wandb.sdk.artifacts.storage_policies.register import WANDB_STORAGE_POLICY
from wandb.sdk.artifacts.storage_policy import StoragePolicy
from wandb.sdk.internal.internal_api import Api as InternalApi
from wandb.sdk.lib.hashutil import b64_to_hex_id, hex_to_b64_id
from wandb.sdk.lib.paths import FilePathStr, URIStr

from ._factories import make_http_session, make_storage_handlers

if TYPE_CHECKING:
    import requests

    from wandb.filesync.step_prepare import StepPrepare
    from wandb.sdk.artifacts.artifact import Artifact
    from wandb.sdk.artifacts.artifact_manifest_entry import ArtifactManifestEntry
    from wandb.sdk.internal import progress

logger = logging.getLogger(__name__)


class WandbStoragePolicy(StoragePolicy):
    @classmethod
    def name(cls) -> str:
        return WANDB_STORAGE_POLICY

    @classmethod
    def from_config(cls, config: StoragePolicyConfig) -> WandbStoragePolicy:
        return cls(config=config)

    def __init__(
        self,
        config: StoragePolicyConfig | None = None,
        cache: ArtifactFileCache | None = None,
        api: InternalApi | None = None,
    ) -> None:
        self._config = StoragePolicyConfig.model_validate(config or {})

        # Don't instantiate these right away if missing, instead defer to the
        # first time they're needed. Otherwise, at the time of writing, this
        # significantly slows down `Artifact.__init__()`.
        self._maybe_cache = cache
        self._maybe_api = api
        self._maybe_session: requests.Session | None = None
        self._maybe_handler: MultiHandler | None = None

    @property
    def _cache(self) -> ArtifactFileCache:
        if self._maybe_cache is None:
            self._maybe_cache = get_artifact_file_cache()
        return self._maybe_cache

    @property
    def _api(self) -> InternalApi:
        if self._maybe_api is None:
            self._maybe_api = InternalApi()
        return self._maybe_api

    @_api.setter
    def _api(self, api: InternalApi) -> None:
        self._maybe_api = api

    @property
    def _session(self) -> requests.Session:
        if self._maybe_session is None:
            self._maybe_session = make_http_session()
        return self._maybe_session

    @property
    def _handler(self) -> MultiHandler:
        if self._maybe_handler is None:
            self._maybe_handler = MultiHandler(
                handlers=make_storage_handlers(self._session),
                default_handler=TrackingHandler(),
            )
        return self._maybe_handler

    def config(self) -> dict[str, Any]:
        return self._config.model_dump(exclude_none=True)

    def load_file(
        self,
        artifact: Artifact,
        manifest_entry: ArtifactManifestEntry,
        dest_path: str | None = None,
        # FIXME: We should avoid passing the executor into multiple inner functions,
        # it leads to confusing code and opaque tracebacks/call stacks.
        executor: concurrent.futures.Executor | None = None,
    ) -> FilePathStr:
        """Use cache or download the file using signed url.

        Args:
            executor: A thread pool provided by the caller for multi-file
                downloads. Reuse the thread pool for multipart downloads; it is
                closed when the artifact download completes. If this is `None`,
                download the file serially.
        """
        if dest_path is not None:
            self._cache._override_cache_path = dest_path

        path, hit, cache_open = self._cache.check_md5_obj_path(
            manifest_entry.digest,
            size=manifest_entry.size or 0,
        )
        if hit:
            return path

        if url := manifest_entry._download_url:
            # Use multipart parallel download for large file
            if executor and (size := manifest_entry.size):
                multipart_download(executor, self._session, url, size, cache_open)
                return path

            # Serial download
            try:
                response = self._session.get(url, stream=True)
            except requests.HTTPError:
                # Signed URL might have expired, fall back to fetching it one by one.
                manifest_entry._download_url = None

        if manifest_entry._download_url is None:
            auth = None
            headers: dict[str, str] = {}

            # For auth, prefer using (in order): auth header, cookies, HTTP Basic Auth
            if token := self._api.access_token:
                headers = {"Authorization": f"Bearer {token}"}
            else:
                auth = ("api", self._api.api_key or "")

            file_url = self._file_url(artifact, manifest_entry)
            response = self._session.get(
                file_url,
                auth=auth,
                headers=headers,
                stream=True,
            )

        with cache_open(mode="wb") as file:
            for data in response.iter_content(chunk_size=16 * KiB):
                file.write(data)
        return path

    def store_reference(
        self,
        artifact: Artifact,
        path: URIStr | FilePathStr,
        name: str | None = None,
        checksum: bool = True,
        max_objects: int | None = None,
    ) -> list[ArtifactManifestEntry]:
        return self._handler.store_path(
            artifact, path, name=name, checksum=checksum, max_objects=max_objects
        )

    def load_reference(
        self,
        manifest_entry: ArtifactManifestEntry,
        local: bool = False,
        dest_path: str | None = None,
    ) -> FilePathStr | URIStr:
        assert manifest_entry.ref is not None
        used_handler = self._handler._get_handler(manifest_entry.ref)
        if hasattr(used_handler, "_cache") and (dest_path is not None):
            used_handler._cache._override_cache_path = dest_path
        return self._handler.load_path(manifest_entry, local)

    def _file_url(self, artifact: Artifact, entry: ArtifactManifestEntry) -> str:
        api = self._api
        base_url: str = api.settings("base_url")

        layout = self._config.storage_layout or StorageLayout.V1
        region = self._config.storage_region or "default"

        entity = artifact.entity
        project = artifact.project
        collection = artifact.name.split(":")[0]

        hexhash = b64_to_hex_id(entry.digest)

        if layout is StorageLayout.V1:
            return f"{base_url}/artifacts/{entity}/{hexhash}"

        if layout is StorageLayout.V2:
            birth_artifact_id = entry.birth_artifact_id or ""
            if server_supports(
                api.client, pb.ARTIFACT_COLLECTION_MEMBERSHIP_FILE_DOWNLOAD_HANDLER
            ):
                return f"{base_url}/artifactsV2/{region}/{quote(entity)}/{quote(project)}/{quote(collection)}/{quote(birth_artifact_id)}/{hexhash}/{entry.path.name}"

            return f"{base_url}/artifactsV2/{region}/{quote(entity)}/{quote(birth_artifact_id)}/{hexhash}"

        assert_never(layout)

    def s3_multipart_file_upload(
        self,
        file_path: str,
        chunk_size: int,
        hex_digests: dict[int, str],
        multipart_urls: dict[int, str],
        extra_headers: dict[str, str],
    ) -> list[dict[str, Any]]:
        etags: deque[dict[str, Any]] = deque()
        file_chunks = scan_chunks(file_path, chunk_size)
        for num, data in enumerate(file_chunks, start=1):
            rsp = self._api.upload_multipart_file_chunk_retry(
                multipart_urls[num],
                data,
                extra_headers={
                    "content-md5": hex_to_b64_id(hex_digests[num]),
                    "content-length": str(len(data)),
                    "content-type": extra_headers.get("Content-Type") or "",
                },
            )
            assert rsp is not None
            etags.append({"partNumber": num, "hexMD5": rsp.headers["ETag"]})
        return list(etags)

    def default_file_upload(
        self,
        upload_url: str,
        file_path: str,
        extra_headers: dict[str, Any],
        progress_callback: progress.ProgressFn | None = None,
    ) -> None:
        """Upload a file to the artifact store and write to cache."""
        with open(file_path, "rb") as file:
            # This fails if we don't send the first byte before the signed URL expires.
            self._api.upload_file_retry(
                upload_url, file, progress_callback, extra_headers=extra_headers
            )

    def store_file(
        self,
        artifact_id: str,
        artifact_manifest_id: str,
        entry: ArtifactManifestEntry,
        preparer: StepPrepare,
        progress_callback: progress.ProgressFn | None = None,
    ) -> bool:
        """Upload a file to the artifact store.

        Returns:
            True if the file was a duplicate (did not need to be uploaded),
            False if it needed to be uploaded or was a reference (nothing to dedupe).
        """
        file_size = entry.size or 0
        chunk_size = calc_part_size(file_size)
        file_path = entry.local_path or ""
        # Logic for AWS s3 multipart upload.
        # Only chunk files if larger than 2 GiB. Currently can only support up to 5TiB.
        if MIN_MULTI_UPLOAD_SIZE <= file_size <= MAX_MULTI_UPLOAD_SIZE:
            file_chunks = scan_chunks(file_path, chunk_size)
            upload_parts = [
                {"partNumber": num, "hexMD5": hashlib.md5(data).hexdigest()}
                for num, data in enumerate(file_chunks, start=1)
            ]
            hex_digests = dict(map(itemgetter("partNumber", "hexMD5"), upload_parts))
        else:
            upload_parts = []
            hex_digests = {}

        resp = preparer.prepare(
            {
                "artifactID": artifact_id,
                "artifactManifestID": artifact_manifest_id,
                "name": entry.path,
                "md5": entry.digest,
                "uploadPartsInput": upload_parts,
            }
        ).get()

        entry.birth_artifact_id = resp.birth_artifact_id

        if resp.upload_url is None:
            return True
        if entry.local_path is None:
            return False

        extra_headers = dict(hdr.split(":", 1) for hdr in (resp.upload_headers or []))

        # This multipart upload isn't available, do a regular single url upload
        if (multipart_urls := resp.multipart_upload_urls) is None and resp.upload_url:
            self.default_file_upload(
                resp.upload_url, file_path, extra_headers, progress_callback
            )
        elif multipart_urls is None:
            raise ValueError(f"No multipart urls to upload for file: {file_path}")
        else:
            # Upload files using s3 multipart upload urls
            etags = self.s3_multipart_file_upload(
                file_path,
                chunk_size,
                hex_digests,
                multipart_urls,
                extra_headers,
            )
            assert resp.storage_path is not None
            self._api.complete_multipart_upload_artifact(
                artifact_id, resp.storage_path, etags, resp.upload_id
            )
        self._write_cache(entry)

        return False

    def _write_cache(self, entry: ArtifactManifestEntry) -> None:
        if entry.local_path is None:
            return

        # Cache upon successful upload.
        _, hit, cache_open = self._cache.check_md5_obj_path(
            entry.digest,
            size=entry.size or 0,
        )

        staging_dir = get_staging_dir()
        try:
            if not (entry.skip_cache or hit):
                with cache_open("wb") as f, open(entry.local_path, "rb") as src:
                    shutil.copyfileobj(src, f)
            if entry.local_path.startswith(staging_dir):
                # Delete staged files here instead of waiting till
                # all the files are uploaded
                os.chmod(entry.local_path, 0o600)
                os.remove(entry.local_path)
        except OSError as e:
            termwarn(f"Failed to cache {entry.local_path}, ignoring {e}")



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import RegistryFragment


class UpsertRegistry(GQLResult):
    upsert_model: Optional[UpsertRegistryUpsertModel] = Field(alias="upsertModel")


class UpsertRegistryUpsertModel(GQLResult):
    inserted: Optional[bool]
    project: Optional[RegistryFragment]


UpsertRegistry.model_rebuild()
UpsertRegistryUpsertModel.model_rebuild()



"""Validation for API keys."""

from __future__ import annotations

import re


def check_api_key(key: str) -> str | None:
    """Returns text describing problems with the API key, or None.

    If the key is in a valid format, returns None. Otherwise, returns
    a string formatted as a complete sentence (capitalized, punctuated)
    explaining the problem with the key.

    Args:
        key: The API key to check.
    """
    if not key:
        return "API key is empty."

    # On-prem API keys have a variable-length prefix followed by a dash.
    #
    # NOTE: This should be rsplit(), but it is split() to be backward compatible
    # with tests that rely on that. It should be safe to change to rsplit()
    # once our tests are updated.
    parts = key.split("-", 1)
    if len(parts) == 1:
        secret = parts[0]
    else:
        _, secret = parts

    # NOTE: Dashes only allowed because of split() instead of rsplit() above.
    if not re.fullmatch(r"[\w-]+", secret):
        return "API key may only contain the letters A-Z, digits and underscores."

    if (secret_len := len(secret)) < 40:
        return f"API key must have 40+ characters, has {secret_len}."

    return None



from __future__ import annotations


class InvalidRepositoryError(Exception):
    """The given string is not a valid repository name."""


def resolve_repository_name(repo_name: str) -> tuple[str, str]:
    if "://" in repo_name:
        raise InvalidRepositoryError(
            f"Repository name cannot contain a scheme ({repo_name})"
        )

    index_name, remote_name = split_repo_name(repo_name)
    if index_name[0] == "-" or index_name[-1] == "-":
        raise InvalidRepositoryError(
            f"Invalid index name ({index_name}). Cannot begin or end with a hyphen."
        )
    return resolve_index_name(index_name), remote_name


def resolve_index_name(index_name: str) -> str:
    index_name = convert_to_hostname(index_name)
    if index_name == "index.docker.io":
        index_name = "docker.io"
    return index_name


def split_repo_name(repo_name: str) -> tuple[str, str]:
    parts = repo_name.split("/", 1)
    if len(parts) == 1 or (
        "." not in parts[0] and ":" not in parts[0] and parts[0] != "localhost"
    ):
        # This is a docker index repo (ex: username/foobar or ubuntu)
        return "docker.io", repo_name
    return parts[0], parts[1]


def convert_to_hostname(url: str) -> str:
    return url.replace("http://", "").replace("https://", "").split("/", 1)[0]



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from datetime import datetime
from typing import List, Optional, Union

from pydantic import Field
from typing_extensions import Literal

from wandb._pydantic import GQLId, GQLResult, Typename

from .enums import AlertSeverity, EventTriggeringConditionType


class ArtifactPortfolioScopeFields(GQLResult):
    typename__: Typename[Literal["ArtifactPortfolio"]] = "ArtifactPortfolio"
    id: GQLId
    name: str


class ArtifactSequenceScopeFields(GQLResult):
    typename__: Typename[Literal["ArtifactSequence"]] = "ArtifactSequence"
    id: GQLId
    name: str


class FilterEventFields(GQLResult):
    typename__: Typename[Literal["FilterEventTriggeringCondition"]] = (
        "FilterEventTriggeringCondition"
    )
    event_type: EventTriggeringConditionType = Field(alias="eventType")
    filter: str


class WebhookIntegrationFields(GQLResult):
    typename__: Typename[Literal["GenericWebhookIntegration"]] = (
        "GenericWebhookIntegration"
    )
    id: GQLId
    name: str
    url_endpoint: str = Field(alias="urlEndpoint")


class GenericWebhookActionFields(GQLResult):
    typename__: Typename[Literal["GenericWebhookTriggeredAction"]] = (
        "GenericWebhookTriggeredAction"
    )
    integration: Union[
        GenericWebhookActionFieldsIntegrationIntegration, WebhookIntegrationFields
    ] = Field(discriminator="typename__")
    request_payload: Optional[str] = Field(alias="requestPayload")


class GenericWebhookActionFieldsIntegrationIntegration(GQLResult):
    typename__: Typename[
        Literal["GitHubOAuthIntegration", "Integration", "SlackIntegration"]
    ]


class NoOpActionFields(GQLResult):
    typename__: Typename[Literal["NoOpTriggeredAction"]] = "NoOpTriggeredAction"
    no_op: Optional[bool] = Field(alias="noOp")


class SlackIntegrationFields(GQLResult):
    typename__: Typename[Literal["SlackIntegration"]] = "SlackIntegration"
    id: GQLId
    team_name: str = Field(alias="teamName")
    channel_name: str = Field(alias="channelName")


class NotificationActionFields(GQLResult):
    typename__: Typename[Literal["NotificationTriggeredAction"]] = (
        "NotificationTriggeredAction"
    )
    integration: Union[
        NotificationActionFieldsIntegrationIntegration, SlackIntegrationFields
    ] = Field(discriminator="typename__")
    title: Optional[str]
    message: Optional[str]
    severity: Optional[AlertSeverity]


class NotificationActionFieldsIntegrationIntegration(GQLResult):
    typename__: Typename[
        Literal["GenericWebhookIntegration", "GitHubOAuthIntegration", "Integration"]
    ]


class PageInfoFields(GQLResult):
    end_cursor: Optional[str] = Field(alias="endCursor")
    has_next_page: bool = Field(alias="hasNextPage")


class ProjectScopeFields(GQLResult):
    typename__: Typename[Literal["Project"]] = "Project"
    id: GQLId
    name: str


class QueueJobActionFields(GQLResult):
    typename__: Typename[Literal["QueueJobTriggeredAction"]] = "QueueJobTriggeredAction"
    queue: Optional[QueueJobActionFieldsQueue]
    template: str


class QueueJobActionFieldsQueue(GQLResult):
    id: GQLId
    name: str


class TriggerFields(GQLResult):
    typename__: Typename[Literal["Trigger"]] = "Trigger"
    id: GQLId
    created_at: datetime = Field(alias="createdAt")
    updated_at: Optional[datetime] = Field(alias="updatedAt")
    name: str
    description: Optional[str]
    enabled: bool
    scope: Union[
        ProjectScopeFields, ArtifactSequenceScopeFields, ArtifactPortfolioScopeFields
    ] = Field(discriminator="typename__")
    event: FilterEventFields
    action: Union[
        QueueJobActionFields,
        NotificationActionFields,
        GenericWebhookActionFields,
        NoOpActionFields,
    ] = Field(discriminator="typename__")


class ProjectTriggersFields(GQLResult):
    typename__: Typename[Literal["Project"]] = "Project"
    triggers: List[TriggerFields]


ArtifactPortfolioScopeFields.model_rebuild()
ArtifactSequenceScopeFields.model_rebuild()
FilterEventFields.model_rebuild()
WebhookIntegrationFields.model_rebuild()
GenericWebhookActionFields.model_rebuild()
NoOpActionFields.model_rebuild()
SlackIntegrationFields.model_rebuild()
NotificationActionFields.model_rebuild()
PageInfoFields.model_rebuild()
ProjectScopeFields.model_rebuild()
QueueJobActionFields.model_rebuild()
TriggerFields.model_rebuild()
ProjectTriggersFields.model_rebuild()



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import FileWithUrlFragment, PageInfoFragment


class GetArtifactFileUrls(GQLResult):
    artifact: Optional[GetArtifactFileUrlsArtifact]


class GetArtifactFileUrlsArtifact(GQLResult):
    files: Optional[GetArtifactFileUrlsArtifactFiles]


class GetArtifactFileUrlsArtifactFiles(GQLResult):
    page_info: PageInfoFragment = Field(alias="pageInfo")
    edges: List[GetArtifactFileUrlsArtifactFilesEdges]


class GetArtifactFileUrlsArtifactFilesEdges(GQLResult):
    node: Optional[FileWithUrlFragment]


GetArtifactFileUrls.model_rebuild()
GetArtifactFileUrlsArtifact.model_rebuild()
GetArtifactFileUrlsArtifactFiles.model_rebuild()
GetArtifactFileUrlsArtifactFilesEdges.model_rebuild()



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactMembershipFragment


class LinkArtifact(GQLResult):
    result: Optional[LinkArtifactResult]


class LinkArtifactResult(GQLResult):
    version_index: Optional[int] = Field(alias="versionIndex")
    artifact_membership: Optional[ArtifactMembershipFragment] = Field(
        alias="artifactMembership", default=None
    )


LinkArtifact.model_rebuild()
LinkArtifactResult.model_rebuild()



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactCollectionFragment


class UpdateArtifactSequenceType(GQLResult):
    result: Optional[UpdateArtifactSequenceTypeResult]


class UpdateArtifactSequenceTypeResult(GQLResult):
    artifact_collection: Optional[ArtifactCollectionFragment] = Field(
        alias="artifactCollection"
    )


UpdateArtifactSequenceType.model_rebuild()
UpdateArtifactSequenceTypeResult.model_rebuild()



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactTypeFragment, PageInfoFragment


class ProjectArtifactTypes(GQLResult):
    project: Optional[ProjectArtifactTypesProject]


class ProjectArtifactTypesProject(GQLResult):
    artifact_types: ProjectArtifactTypesProjectArtifactTypes = Field(
        alias="artifactTypes"
    )


class ProjectArtifactTypesProjectArtifactTypes(GQLResult):
    edges: List[ProjectArtifactTypesProjectArtifactTypesEdges]
    page_info: PageInfoFragment = Field(alias="pageInfo")


class ProjectArtifactTypesProjectArtifactTypesEdges(GQLResult):
    node: Optional[ArtifactTypeFragment]


ProjectArtifactTypes.model_rebuild()
ProjectArtifactTypesProject.model_rebuild()
ProjectArtifactTypesProjectArtifactTypes.model_rebuild()
ProjectArtifactTypesProjectArtifactTypesEdges.model_rebuild()



"""W&B Public API for Automation objects."""

from __future__ import annotations

from itertools import chain
from typing import TYPE_CHECKING, Any, Iterator, Mapping

from pydantic import ValidationError
from typing_extensions import override

from wandb.apis.paginator import RelayPaginator, _Client

if TYPE_CHECKING:
    from wandb_graphql.language.ast import Document

    from wandb._pydantic import Connection
    from wandb.automations import Automation
    from wandb.automations._generated import ProjectTriggersFields


class Automations(RelayPaginator["ProjectTriggersFields", "Automation"]):
    """A lazy iterator of `Automation` objects.

    <!-- lazydoc-ignore-class: internal -->
    """

    QUERY: Document  # Must be set per-instance
    last_response: Connection[ProjectTriggersFields] | None

    def __init__(
        self,
        client: _Client,
        variables: Mapping[str, Any],
        per_page: int = 50,
        *,
        _query: Document,  # internal use only, but required
    ):
        self.QUERY = _query
        super().__init__(client, variables=variables, per_page=per_page)

    @override
    def _update_response(self) -> None:
        """Fetch the raw response data for the current page."""
        from wandb._pydantic import Connection
        from wandb.automations._generated import ProjectTriggersFields

        data = self.client.execute(self.QUERY, variable_values=self.variables)
        try:
            conn_data = data["scope"]["projects"]
            conn = Connection[ProjectTriggersFields].model_validate(conn_data)
            self.last_response = conn
        except (LookupError, AttributeError, ValidationError) as e:
            raise ValueError("Unexpected response data") from e

    @override
    def _convert(self, node: ProjectTriggersFields) -> Iterator[Automation]:
        from wandb.automations import Automation

        return (Automation.model_validate(obj) for obj in node.triggers)

    @override
    def convert_objects(self) -> Iterator[Automation]:
        return chain.from_iterable(super().convert_objects())



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import PageInfoFields, ProjectTriggersFields


class GetAutomations(GQLResult):
    scope: Optional[GetAutomationsScope]


class GetAutomationsScope(GQLResult):
    projects: Optional[GetAutomationsScopeProjects]


class GetAutomationsScopeProjects(GQLResult):
    page_info: PageInfoFields = Field(alias="pageInfo")
    edges: List[GetAutomationsScopeProjectsEdges]


class GetAutomationsScopeProjectsEdges(GQLResult):
    node: Optional[ProjectTriggersFields]


GetAutomations.model_rebuild()
GetAutomationsScope.model_rebuild()
GetAutomationsScopeProjects.model_rebuild()
GetAutomationsScopeProjectsEdges.model_rebuild()



from __future__ import annotations

from base64 import b64decode, b64encode
from typing import Any


def removeprefix(s: str, prefix: str) -> str:
    """Removes a prefix from a string.

    This roughly backports the built-in `str.removeprefix` function from Python 3.9+.
    Once Python 3.8 support is dropped, just replace this with `str.removeprefix`.
    """
    return s[len(prefix) :] if s.startswith(prefix) else s


def removesuffix(s: str, suffix: str) -> str:
    """Removes a suffix from a string.

    This roughly backports the built-in `str.removesuffix` function from Python 3.9+.
    Once Python 3.8 support is dropped, just replace this with `str.removesuffix`.
    """
    return s[: -len(suffix)] if s.endswith(suffix) else s


def ensureprefix(s: str, prefix: str) -> str:
    """Ensures the string has the given prefix prepended."""
    return s if s.startswith(prefix) else f"{prefix}{s}"


def ensuresuffix(s: str, suffix: str) -> str:
    """Ensures the string has the given suffix appended."""
    return s if s.endswith(suffix) else f"{s}{suffix}"


def nameof(obj: Any, full: bool = True) -> str:
    """Internal convenience helper that returns the object's `__name__` or `__qualname__`.

    If `full` is True, attempt to return the object's `__qualname__` attribute,
    falling back on the `__name__` attribute.
    """
    return getattr(obj, "__qualname__", obj.__name__) if full else obj.__name__


def b64decode_ascii(s: str) -> str:
    """Returns the decoded base64 string interpreted as ASCII.

    Convenience function for directly converting `str -> str`.
    """
    return b64decode(s).decode("ascii")


def b64encode_ascii(s: str) -> str:
    """Returns the base64 encoding of the string's ASCII bytes.

    Convenience function for directly converting `str -> str`.
    """
    return b64encode(s.encode("ascii")).decode("ascii")



"""Module for starting up the service process (wandb-core)."""

from __future__ import annotations

import os
import pathlib
import platform
import subprocess
import tempfile
from typing import TYPE_CHECKING

from wandb.analytics import get_sentry
from wandb.env import core_debug, dcgm_profiling_enabled, error_reporting_enabled
from wandb.errors import WandbCoreNotAvailableError
from wandb.sdk.lib.service import ipc_support
from wandb.util import get_core_path

from . import service_port_file, service_token

if TYPE_CHECKING:
    from wandb.sdk.wandb_settings import Settings


def start(settings: Settings) -> ServiceProcess:
    """Start the internal service process.

    Returns:
        A handle to the process.
    """
    get_sentry().configure_scope(tags=dict(settings), process_context="service")

    try:
        return _launch_server(settings)
    except Exception as e:
        get_sentry().reraise(e)


class ServiceProcess:
    """A handle to a process running the internal service."""

    def __init__(
        self,
        *,
        connection_token: service_token.ServiceToken,
        process: subprocess.Popen,
    ) -> None:
        self._token = connection_token
        self._process = process

    @property
    def token(self) -> service_token.ServiceToken:
        """A token for connecting to the process."""
        return self._token

    def join(self) -> int:
        """Wait for the process to end and return its exit code."""
        return self._process.wait()


def _launch_server(settings: Settings) -> ServiceProcess:
    """Launch server and set ports."""
    if platform.system() == "Windows":
        creationflags: int = subprocess.CREATE_NEW_PROCESS_GROUP  # type: ignore[attr-defined]
        start_new_session = False
    else:
        creationflags = 0
        start_new_session = True

    pid = str(os.getpid())

    with tempfile.TemporaryDirectory() as tmpdir:
        port_file = pathlib.Path(tmpdir, f"port-{pid}.txt")
        service_args: list[str] = []

        try:
            core_path = get_core_path()
        except WandbCoreNotAvailableError as e:
            get_sentry().reraise(e)

        service_args.extend([core_path])

        if not error_reporting_enabled():
            service_args.append("--no-observability")

        if core_debug(default="False"):
            service_args.extend(["--log-level", "-4"])

        if dcgm_profiling_enabled():
            service_args.append("--enable-dcgm-profiling")

        service_args.extend(["--port-filename", str(port_file)])
        service_args.extend(["--pid", pid])

        if not ipc_support.SUPPORTS_UNIX:
            service_args.append("--listen-on-localhost")

        proc = subprocess.Popen(
            service_args,
            env=os.environ,
            close_fds=True,
            creationflags=creationflags,
            start_new_session=start_new_session,
        )

        token = service_port_file.poll_for_token(
            port_file,
            proc,
            timeout=settings.x_service_wait,
        )

        return ServiceProcess(connection_token=token, process=proc)



from __future__ import annotations

from typing import TYPE_CHECKING

from typing_extensions import TypeAlias

import wandb
from wandb.sdk.lib import telemetry

if TYPE_CHECKING:
    from wandb.proto.wandb_telemetry_pb2 import Deprecated


UNSET: DoNotSet = object()
"""A temporary default value for removing formerly optional parameters.

This helps distinguish explicit None arguments from implicit None.

Usage:
    def myfunction(old_param: DoNotSet = UNSET) -> ...:
        if old_param is not UNSET:
            warn_user()
        else:
            # Set to None if checked in downstream logic,
            # since UNSET is not None.
            old_param = None
        ...

    myfunction()                # Does not warn
    myfunction(old_param=None)  # Warns
"""

DoNotSet: TypeAlias = object
"""The type of UNSET."""


def warn_and_record_deprecation(
    *,
    feature: Deprecated,
    message: str,
    run: wandb.Run | None = None,
) -> None:
    """Warn the user that a feature has been deprecated and update telemetry.

    Args:
        feature: A Deprecated protobuf message with the relevant field set to True.
        message: The deprecation warning message to display to the user.
        run: The run whose telemetry to update.
    """
    with telemetry.context(run=run or wandb.run) as tel:
        tel.deprecated.MergeFrom(feature)

    wandb.termwarn(message, repeat=False)



"""A message protocol for the internal service process.

The core of W&B is implemented by a side process that asynchronously uploads
data. The client process (such as this Python code) sends requests to the
service, and for some requests, the service eventually sends a response.

The client can send multiple requests before the service provides a response.
The Mailbox handles matching responses to requests. An internal thread
continuously reads data from the service and passes it to the mailbox.
"""

from .mailbox import Mailbox, MailboxClosedError
from .mailbox_handle import HandleAbandonedError, MailboxHandle
from .wait_with_progress import wait_all_with_progress, wait_with_progress

__all__ = [
    "Mailbox",
    "MailboxClosedError",
    "HandleAbandonedError",
    "MailboxHandle",
    "wait_all_with_progress",
    "wait_with_progress",
]



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from typing_extensions import override

from wandb.proto import wandb_server_pb2 as spb
from wandb.sdk.lib import asyncio_manager

from .interface_shared import InterfaceShared

if TYPE_CHECKING:
    from wandb.proto import wandb_internal_pb2 as pb
    from wandb.sdk.lib.service.service_client import ServiceClient
    from wandb.sdk.mailbox import MailboxHandle


logger = logging.getLogger("wandb")


class InterfaceSock(InterfaceShared):
    def __init__(
        self,
        asyncer: asyncio_manager.AsyncioManager,
        client: ServiceClient,
        stream_id: str,
    ) -> None:
        super().__init__()
        self._asyncer = asyncer
        self._client = client
        self._stream_id = stream_id

    def _assign(self, record: pb.Record) -> None:
        record._info.stream_id = self._stream_id

    @override
    def _publish(self, record: pb.Record, *, nowait: bool = False) -> None:
        self._assign(record)
        request = spb.ServerRequest()
        request.record_publish.CopyFrom(record)

        if nowait:
            self._asyncer.run_soon(lambda: self._client.publish(request))
        else:
            self._asyncer.run(lambda: self._client.publish(request))

    @override
    def _deliver(self, record: pb.Record) -> MailboxHandle[pb.Result]:
        return self._asyncer.run(lambda: self.deliver_async(record))

    @override
    async def deliver_async(self, record: pb.Record) -> MailboxHandle[pb.Result]:
        self._assign(record)
        request = spb.ServerRequest()
        request.record_publish.CopyFrom(record)

        handle = await self._client.deliver(request)
        return handle.map(lambda response: response.result_communicate)



"""Helpers and constants for multipart upload and download."""

from __future__ import annotations

import logging
import math
import threading
from concurrent.futures import FIRST_EXCEPTION, Executor, wait
from dataclasses import dataclass, field
from queue import Queue
from typing import TYPE_CHECKING, Any, Final, Iterator, Union

from typing_extensions import TypeAlias, TypeIs, final

from wandb import env
from wandb.sdk.artifacts.artifact_file_cache import Opener

if TYPE_CHECKING:
    from requests import Session

logger = logging.getLogger(__name__)

KiB: Final[int] = 1024
MiB: Final[int] = 1024**2
GiB: Final[int] = 1024**3
TiB: Final[int] = 1024**4

# AWS S3 max upload parts without having to make additional requests for extra parts
MAX_PARTS = 1_000
MIN_MULTI_UPLOAD_SIZE = 2 * GiB
MAX_MULTI_UPLOAD_SIZE = 5 * TiB

# Minimum size to switch to multipart download, same threshold as upload.
MIN_MULTI_DOWNLOAD_SIZE = MIN_MULTI_UPLOAD_SIZE

# Multipart download part size matches the upload size and is hard coded to
# 100 MB.
# https://github.com/wandb/wandb/blob/7b2a13cb8efcd553317167b823c8e52d8c3f7c4e/core/pkg/artifacts/saver.go#L496
# https://docs.aws.amazon.com/AmazonS3/latest/userguide/optimizing-performance-guidelines.html#optimizing-performance-guidelines-get-range
MULTI_DEFAULT_PART_SIZE = 100 * MiB

# Chunk size for reading http response and writing to disk.
RSP_CHUNK_SIZE = 1 * MiB


@final
class _ChunkSentinel:
    """Signal the end of the multipart chunk queue.

    Queue consumers terminate when they receive this item from the queue. Do
    not instantiate this class directly; use the `END_CHUNK` constant as a
    pseudo-singleton instead.

    NOTE: Use this only in multi-threaded (not multi-process) contexts because
    it is not guaranteed to be process-safe.
    """

    def __repr__(self) -> str:
        return "ChunkSentinel"


END_CHUNK: Final[_ChunkSentinel] = _ChunkSentinel()


def is_end_chunk(obj: Any) -> TypeIs[_ChunkSentinel]:
    """Returns True if the object is the terminal queue item for multipart downloads."""
    # Needed for type checking, since _ChunkSentinel isn't formally a singleton.
    return obj is END_CHUNK


@dataclass(frozen=True)
class ChunkContent:
    __slots__ = ("offset", "data")  # slots=True only introduced in Python 3.10
    offset: int
    data: bytes


QueuedChunk: TypeAlias = Union[ChunkContent, _ChunkSentinel]


def should_multipart_download(size: int | None, override: bool | None = None) -> bool:
    return ((size or 0) >= MIN_MULTI_DOWNLOAD_SIZE) if (override is None) else override


def calc_part_size(file_size: int, min_part_size: int = MULTI_DEFAULT_PART_SIZE) -> int:
    # Default to a chunk size of 100MiB. S3 has a cap of 10,000 upload parts.
    return max(math.ceil(file_size / MAX_PARTS), min_part_size)


def scan_chunks(path: str, chunk_size: int) -> Iterator[bytes]:
    with open(path, "rb") as f:
        while data := f.read(chunk_size):
            yield data


@dataclass
class MultipartDownloadContext:
    q: Queue[QueuedChunk]
    cancel: threading.Event = field(default_factory=threading.Event)


def multipart_download(
    executor: Executor,
    session: Session,
    url: str,
    size: int,
    cached_open: Opener,
    part_size: int = MULTI_DEFAULT_PART_SIZE,
):
    """Download file as multiple parts in parallel.

    Only one thread for writing to file. Each part run one http request in one thread.
    HTTP response chunk of a file part is sent to the writer thread via a queue.
    """
    # ------------------------------------------------------------------------------
    # Shared between threads
    ctx = MultipartDownloadContext(q=Queue(maxsize=500))

    # Put cache_open at top so we remove the tmp file when there is network error.
    with cached_open("wb") as f:

        def download_chunk(start: int, end: int | None = None) -> None:
            # Error from another thread, no need to start
            if ctx.cancel.is_set():
                return

            # https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Headers/Range
            # Start and end are both inclusive. An empty end uses the actual end
            # of the file. Example string: "bytes=0-499".
            bytes_range = f"{start}-" if (end is None) else f"{start}-{end}"
            headers = {"Range": f"bytes={bytes_range}"}
            with session.get(url=url, headers=headers, stream=True) as rsp:
                offset = start
                for chunk in rsp.iter_content(chunk_size=RSP_CHUNK_SIZE):
                    if ctx.cancel.is_set():
                        return
                    ctx.q.put(ChunkContent(offset=offset, data=chunk))
                    offset += len(chunk)

        def write_chunks() -> None:
            # If all chunks are written or there's an error in another thread, shutdown
            while not (ctx.cancel.is_set() or is_end_chunk(chunk := ctx.q.get())):
                try:
                    # NOTE: Seek works without pre allocating the file on disk.
                    # It automatically creates a sparse file, e.g. ls -hl would show
                    # a bigger size compared to du -sh * because downloading different
                    # chunks is not a sequential write.
                    # See https://man7.org/linux/man-pages/man2/lseek.2.html
                    f.seek(chunk.offset)
                    f.write(chunk.data)

                except Exception as e:
                    if env.is_debug():
                        logger.debug(f"Error writing chunk to file: {e}")
                    ctx.cancel.set()
                    raise

        # Start writer thread first.
        write_future = executor.submit(write_chunks)

        # Start download threads for each chunk.
        download_futures = set()
        for start in range(0, size, part_size):
            # https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Headers/Range
            # Start and end are both inclusive. An empty end uses the actual end
            # of the file. Example string: "bytes=0-499".
            end = end if (end := (start + part_size - 1)) < size else None
            download_futures.add(executor.submit(download_chunk, start=start, end=end))

        # Wait for download
        done, not_done = wait(download_futures, return_when=FIRST_EXCEPTION)
        try:
            for fut in done:
                fut.result()
        except Exception as e:
            if env.is_debug():
                logger.debug(f"Error downloading file: {e}")
            ctx.cancel.set()

            # Cancel any pending futures.  Note:
            # - `Future.cancel()` does NOT stop the future if it's running, which is why
            #   there's a separate `threading.Event` to ensure cooperative cancellation.
            # - Once Python 3.8 support is dropped, replace these `fut.cancel()`
            #   calls with `Executor.shutdown(cancel_futures=True)`.
            for fut in not_done:
                fut.cancel()
            raise
        finally:
            # Always signal the writer to stop
            ctx.q.put(END_CHUNK)
            write_future.result()



from __future__ import annotations

from typing_extensions import final, override

from wandb.sdk.lib import urls


@final
class HostUrl:
    """A validated and normalized W&B server URL.

    For convenient formatting, the __str__ representation is the URL itself.
    """

    def __init__(self, url: str) -> None:
        urls.validate_url(url)
        self._url = url.rstrip("/")

    def is_same_url(self, value: str | HostUrl, /) -> bool:
        """Compare normalized URLs.

        Returns true if the value is an equivalent HostUrl or a string
        that normalizes to this URL.
        """
        if isinstance(value, HostUrl):
            return self._url == value._url
        else:
            return self._url == value.rstrip("/")

    @property
    def url(self) -> str:
        return self._url

    @override
    def __str__(self) -> str:
        return self._url

    @override
    def __repr__(self) -> str:
        return f"HostUrl({self._url!r})"



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field
from typing_extensions import Literal

from wandb._pydantic import GQLId, GQLResult, Typename

from .enums import ArtifactState


class ArtifactAliasFragment(GQLResult):
    typename__: Typename[Literal["ArtifactAlias"]] = "ArtifactAlias"
    id: GQLId
    alias: str


class ProjectInfoFragment(GQLResult):
    name: str
    entity: ProjectInfoFragmentEntity


class ProjectInfoFragmentEntity(GQLResult):
    name: str


class TagFragment(GQLResult):
    typename__: Typename[Literal["Tag"]] = "Tag"
    id: GQLId
    name: str


class ArtifactCollectionFragment(GQLResult):
    typename__: Typename[Literal["ArtifactSequence", "ArtifactPortfolio"]]
    id: GQLId
    name: str
    description: Optional[str]
    created_at: str = Field(alias="createdAt")
    project: Optional[ProjectInfoFragment]
    type: ArtifactCollectionFragmentType
    tags: ArtifactCollectionFragmentTags


class ArtifactCollectionFragmentType(GQLResult):
    name: str


class ArtifactCollectionFragmentTags(GQLResult):
    edges: List[ArtifactCollectionFragmentTagsEdges]


class ArtifactCollectionFragmentTagsEdges(GQLResult):
    node: TagFragment


class CollectionInfoFragment(GQLResult):
    typename__: Typename[Literal["ArtifactSequence", "ArtifactPortfolio"]]
    name: str
    project: Optional[ProjectInfoFragment]


class SourceCollectionInfoFragment(GQLResult):
    typename__: Typename[Literal["ArtifactSequence"]] = "ArtifactSequence"
    name: str
    project: Optional[ProjectInfoFragment]


class ArtifactFragment(GQLResult):
    typename__: Typename[Literal["Artifact"]] = "Artifact"
    id: GQLId
    artifact_sequence: SourceCollectionInfoFragment = Field(alias="artifactSequence")
    version_index: Optional[int] = Field(alias="versionIndex")
    artifact_type: ArtifactFragmentArtifactType = Field(alias="artifactType")
    description: Optional[str]
    metadata: Optional[str]
    ttl_duration_seconds: Optional[int] = Field(
        alias="ttlDurationSeconds", default=None
    )
    ttl_is_inherited: Optional[bool] = Field(alias="ttlIsInherited", default=None)
    tags: Optional[List[TagFragment]] = None
    history_step: Optional[int] = Field(alias="historyStep", default=None)
    state: ArtifactState
    size: int
    digest: str
    commit_hash: Optional[str] = Field(alias="commitHash")
    file_count: int = Field(alias="fileCount")
    created_at: str = Field(alias="createdAt")
    updated_at: Optional[str] = Field(alias="updatedAt")
    aliases: Optional[List[ArtifactFragmentAliases]] = None


class ArtifactFragmentArtifactType(GQLResult):
    name: str


class ArtifactFragmentAliases(ArtifactAliasFragment):
    artifact_collection: Optional[CollectionInfoFragment] = Field(
        alias="artifactCollection"
    )


class ArtifactMembershipFragment(GQLResult):
    typename__: Typename[Literal["ArtifactCollectionMembership"]] = (
        "ArtifactCollectionMembership"
    )
    id: GQLId
    version_index: Optional[int] = Field(alias="versionIndex")
    aliases: List[ArtifactAliasFragment]
    artifact_collection: Optional[CollectionInfoFragment] = Field(
        alias="artifactCollection"
    )
    artifact: Optional[ArtifactFragment]


class ArtifactPortfolioTypeFields(GQLResult):
    typename__: Typename[Literal["ArtifactPortfolio"]] = "ArtifactPortfolio"
    id: GQLId
    name: str


class ArtifactSequenceTypeFields(GQLResult):
    typename__: Typename[Literal["ArtifactSequence"]] = "ArtifactSequence"
    id: GQLId
    name: str


class ArtifactTypeFragment(GQLResult):
    typename__: Typename[Literal["ArtifactType"]] = "ArtifactType"
    id: GQLId
    name: str
    description: Optional[str]
    created_at: str = Field(alias="createdAt")


class DeferredManifestFragment(GQLResult):
    file: DeferredManifestFragmentFile


class DeferredManifestFragmentFile(GQLResult):
    direct_url: str = Field(alias="directUrl")


class FileFragment(GQLResult):
    typename__: Typename[Literal["File"]] = "File"
    id: GQLId
    name: str
    url: Optional[str]
    size_bytes: int = Field(alias="sizeBytes")
    storage_path: Optional[str] = Field(alias="storagePath", default=None)
    mimetype: Optional[str]
    updated_at: Optional[str] = Field(alias="updatedAt")
    digest: Optional[str]
    md_5: Optional[str] = Field(alias="md5")
    direct_url: str = Field(alias="directUrl")


class FileWithUrlFragment(GQLResult):
    typename__: Typename[Literal["File"]] = "File"
    name: str
    direct_url: str = Field(alias="directUrl")


class OrgInfoFragment(GQLResult):
    name: str
    org_entity: Optional[OrgInfoFragmentOrgEntity] = Field(
        alias="orgEntity", default=None
    )


class OrgInfoFragmentOrgEntity(GQLResult):
    name: str


class PageInfoFragment(GQLResult):
    typename__: Typename[Literal["PageInfo"]] = "PageInfo"
    end_cursor: Optional[str] = Field(alias="endCursor")
    has_next_page: bool = Field(alias="hasNextPage")


class RegistryCollectionFragment(GQLResult):
    typename__: Typename[Literal["ArtifactSequence", "ArtifactPortfolio"]]
    id: GQLId
    name: str
    description: Optional[str]
    created_at: str = Field(alias="createdAt")
    project: Optional[ProjectInfoFragment]
    type: RegistryCollectionFragmentType
    tags: RegistryCollectionFragmentTags


class RegistryCollectionFragmentType(GQLResult):
    name: str


class RegistryCollectionFragmentTags(GQLResult):
    edges: List[RegistryCollectionFragmentTagsEdges]


class RegistryCollectionFragmentTagsEdges(GQLResult):
    node: TagFragment


class RegistryFragment(GQLResult):
    typename__: Typename[Literal["Project"]] = "Project"
    id: GQLId
    name: str
    entity: RegistryFragmentEntity
    description: Optional[str]
    created_at: str = Field(alias="createdAt")
    updated_at: Optional[str] = Field(alias="updatedAt")
    access: Optional[str]
    allow_all_artifact_types: bool = Field(alias="allowAllArtifactTypes")
    artifact_types: RegistryFragmentArtifactTypes = Field(alias="artifactTypes")


class RegistryFragmentEntity(GQLResult):
    name: str
    organization: Optional[RegistryFragmentEntityOrganization]


class RegistryFragmentEntityOrganization(GQLResult):
    name: str


class RegistryFragmentArtifactTypes(GQLResult):
    edges: List[RegistryFragmentArtifactTypesEdges]


class RegistryFragmentArtifactTypesEdges(GQLResult):
    node: Optional[RegistryFragmentArtifactTypesEdgesNode]


class RegistryFragmentArtifactTypesEdgesNode(GQLResult):
    name: str


class RegistryRoleFragment(GQLResult):
    name: str


class RunInfoFragment(GQLResult):
    typename__: Typename[Literal["Run"]] = "Run"
    id: GQLId
    name: str
    project: Optional[ProjectInfoFragment]


class TeamMemberFragment(GQLResult):
    typename__: Typename[Literal["Member"]] = "Member"
    id: Optional[str]
    role: Optional[str]
    pending: Optional[bool]
    email: Optional[str]
    username: Optional[str]
    name: str
    photo_url: Optional[str] = Field(alias="photoUrl")
    account_type: Optional[str] = Field(alias="accountType")
    api_key: Optional[str] = Field(alias="apiKey")


class TeamFragment(GQLResult):
    typename__: Typename[Literal["Entity"]] = "Entity"
    id: GQLId
    name: str
    available: Optional[bool]
    photo_url: Optional[str] = Field(alias="photoUrl")
    read_only: Optional[bool] = Field(alias="readOnly")
    read_only_admin: bool = Field(alias="readOnlyAdmin")
    is_team: bool = Field(alias="isTeam")
    private_only: bool = Field(alias="privateOnly")
    storage_bytes: int = Field(alias="storageBytes")
    code_saving_enabled: bool = Field(alias="codeSavingEnabled")
    default_access: str = Field(alias="defaultAccess")
    is_paid: Optional[bool] = Field(alias="isPaid")
    members: List[TeamMemberFragment]


class TeamRegistryMemberFragment(GQLResult):
    team: TeamFragment
    role: RegistryRoleFragment


class TypeInfoFragment(GQLResult):
    name: Optional[str]
    fields: Optional[List[TypeInfoFragmentFields]]
    input_fields: Optional[List[TypeInfoFragmentInputFields]] = Field(
        alias="inputFields"
    )


class TypeInfoFragmentFields(GQLResult):
    name: str
    args: List[TypeInfoFragmentFieldsArgs]


class TypeInfoFragmentFieldsArgs(GQLResult):
    name: str


class TypeInfoFragmentInputFields(GQLResult):
    name: str


class UserRegistryMemberFragment(GQLResult):
    id: GQLId
    name: Optional[str]
    username: Optional[str]
    email: Optional[str]
    role: RegistryRoleFragment


ArtifactAliasFragment.model_rebuild()
ProjectInfoFragment.model_rebuild()
TagFragment.model_rebuild()
ArtifactCollectionFragment.model_rebuild()
CollectionInfoFragment.model_rebuild()
SourceCollectionInfoFragment.model_rebuild()
ArtifactFragment.model_rebuild()
ArtifactMembershipFragment.model_rebuild()
ArtifactPortfolioTypeFields.model_rebuild()
ArtifactSequenceTypeFields.model_rebuild()
ArtifactTypeFragment.model_rebuild()
DeferredManifestFragment.model_rebuild()
FileFragment.model_rebuild()
FileWithUrlFragment.model_rebuild()
OrgInfoFragment.model_rebuild()
PageInfoFragment.model_rebuild()
RegistryCollectionFragment.model_rebuild()
RegistryFragment.model_rebuild()
RegistryRoleFragment.model_rebuild()
RunInfoFragment.model_rebuild()
TeamMemberFragment.model_rebuild()
TeamFragment.model_rebuild()
TeamRegistryMemberFragment.model_rebuild()
TypeInfoFragment.model_rebuild()
UserRegistryMemberFragment.model_rebuild()



# Generated by ariadne-codegen
# Source: core/api/graphql/schemas/schema-latest.graphql

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLId, GQLInput


class UpsertModelInput(GQLInput):
    name: Optional[str] = Field(default=None, max_length=128)
    description: Optional[str] = None
    id: Optional[str] = None
    framework: Optional[str] = None
    entity_name: Optional[str] = Field(alias="entityName", default=None)
    docker_image: Optional[str] = Field(
        alias="dockerImage", default=None, max_length=512
    )
    repo: Optional[str] = Field(default=None, max_length=256)
    access: Optional[str] = None
    views: Optional[str] = None
    is_benchmark: Optional[bool] = Field(alias="isBenchmark", default=None)
    linked_benchmark: Optional[GQLId] = Field(alias="linkedBenchmark", default=None)
    is_published: Optional[bool] = Field(alias="isPublished", default=None)
    owner: Optional[GQLId] = None
    allow_all_artifact_types_in_registry: Optional[bool] = Field(
        alias="allowAllArtifactTypesInRegistry", default=None
    )
    rate_limits: Optional[RateLimitsInput] = Field(alias="rateLimits", default=None)
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)
    artifact_types: Optional[List[ArtifactTypeInput]] = Field(
        alias="artifactTypes", default=None
    )


class RenameProjectInput(GQLInput):
    entity_name: str = Field(alias="entityName")
    old_project_name: str = Field(alias="oldProjectName")
    new_project_name: str = Field(alias="newProjectName")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class RateLimitsInput(GQLInput):
    graphql: Optional[int] = None
    sdk_graphql: Optional[int] = Field(alias="sdkGraphql", default=None)
    filestream_count: Optional[int] = Field(alias="filestreamCount", default=None)
    filestream_size: Optional[int] = Field(alias="filestreamSize", default=None)
    sdk_graphql_query_seconds: Optional[float] = Field(
        alias="sdkGraphqlQuerySeconds", default=None
    )


class ArtifactTypeInput(GQLInput):
    name: str = Field(max_length=128, pattern="^[-\\w]+([ ]*[-.\\w]+)*$")
    description: Optional[str] = None


class UpdateArtifactSequenceInput(GQLInput):
    artifact_sequence_id: GQLId = Field(alias="artifactSequenceID")
    name: Optional[str] = Field(default=None, max_length=128)
    description: Optional[str] = None
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class MoveArtifactSequenceInput(GQLInput):
    artifact_sequence_id: GQLId = Field(alias="artifactSequenceID")
    destination_artifact_type_name: str = Field(alias="destinationArtifactTypeName")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class UpdateArtifactPortfolioInput(GQLInput):
    artifact_portfolio_id: GQLId = Field(alias="artifactPortfolioID")
    name: Optional[str] = Field(default=None, max_length=128)
    description: Optional[str] = None
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class ArtifactAliasInput(GQLInput):
    artifact_collection_name: str = Field(alias="artifactCollectionName")
    alias: str = Field(max_length=128)


class UpdateArtifactInput(GQLInput):
    artifact_id: GQLId = Field(alias="artifactID")
    description: Optional[str] = None
    labels: Optional[str] = None
    aliases: Optional[List[ArtifactAliasInput]] = None
    tags_to_add: Optional[List[TagInput]] = Field(alias="tagsToAdd", default=None)
    tags_to_delete: Optional[List[TagInput]] = Field(alias="tagsToDelete", default=None)
    metadata: Optional[str] = None
    ttl_duration_seconds: Optional[int] = Field(
        alias="ttlDurationSeconds", default=None
    )
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class DeleteArtifactInput(GQLInput):
    artifact_id: GQLId = Field(alias="artifactID")
    delete_aliases: Optional[bool] = Field(alias="deleteAliases", default=False)
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class LinkArtifactInput(GQLInput):
    artifact_id: Optional[GQLId] = Field(alias="artifactID", default=None)
    artifact_portfolio_id: Optional[GQLId] = Field(
        alias="artifactPortfolioID", default=None
    )
    artifact_portfolio_name: Optional[str] = Field(
        alias="artifactPortfolioName", default=None
    )
    entity_name: Optional[str] = Field(alias="entityName", default=None)
    project_name: Optional[str] = Field(alias="projectName", default=None)
    aliases: Optional[List[ArtifactAliasInput]] = None
    client_id: Optional[GQLId] = Field(alias="clientID", default=None)
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class UnlinkArtifactInput(GQLInput):
    artifact_id: GQLId = Field(alias="artifactID")
    artifact_portfolio_id: GQLId = Field(alias="artifactPortfolioID")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class ArtifactCollectionAliasInput(GQLInput):
    alias: str = Field(max_length=128)
    entity_name: str = Field(alias="entityName")
    project_name: str = Field(alias="projectName")
    artifact_collection_name: str = Field(alias="artifactCollectionName")


class AddAliasesInput(GQLInput):
    aliases: List[ArtifactCollectionAliasInput]
    artifact_id: GQLId = Field(alias="artifactID")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class DeleteAliasesInput(GQLInput):
    aliases: List[ArtifactCollectionAliasInput]
    artifact_id: GQLId = Field(alias="artifactID")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class TagInput(GQLInput):
    tag_category_name: Optional[str] = Field(
        alias="tagCategoryName",
        default=None,
        max_length=128,
        pattern="^[-\\w]+([ ]+[-\\w]+)*$",
    )
    tag_name: str = Field(
        alias="tagName", max_length=128, pattern="^[-\\w]+([ ]+[-\\w]+)*$"
    )
    attributes: Optional[str] = None


class CreateArtifactCollectionTagAssignmentsInput(GQLInput):
    entity_name: str = Field(alias="entityName")
    project_name: str = Field(alias="projectName")
    artifact_collection_name: str = Field(alias="artifactCollectionName")
    tags: List[TagInput] = Field(max_length=20)
    client_mutation_id: Optional[str] = Field(alias="clientMutationID", default=None)


class DeleteArtifactCollectionTagAssignmentsInput(GQLInput):
    entity_name: str = Field(alias="entityName")
    project_name: str = Field(alias="projectName")
    artifact_collection_name: str = Field(alias="artifactCollectionName")
    tags: List[TagInput] = Field(max_length=20)
    client_mutation_id: Optional[str] = Field(alias="clientMutationID", default=None)


class CreateProjectMembersInput(GQLInput):
    user_ids: Optional[List[GQLId]] = Field(alias="userIds", default=None)
    team_ids: Optional[List[GQLId]] = Field(alias="teamIds", default=None)
    project_id: GQLId = Field(alias="projectId")


class DeleteProjectMembersInput(GQLInput):
    user_ids: Optional[List[GQLId]] = Field(alias="userIds", default=None)
    team_ids: Optional[List[GQLId]] = Field(alias="teamIds", default=None)
    project_id: GQLId = Field(alias="projectId")


class UpdateProjectMemberInput(GQLInput):
    user_id: GQLId = Field(alias="userId")
    project_id: GQLId = Field(alias="projectId")
    user_project_role: str = Field(alias="userProjectRole")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


class UpdateProjectTeamMemberInput(GQLInput):
    team_id: GQLId = Field(alias="teamId")
    project_id: GQLId = Field(alias="projectId")
    team_project_role: str = Field(alias="teamProjectRole")
    client_mutation_id: Optional[str] = Field(alias="clientMutationId", default=None)


UpsertModelInput.model_rebuild()
UpdateArtifactInput.model_rebuild()
LinkArtifactInput.model_rebuild()
AddAliasesInput.model_rebuild()
DeleteAliasesInput.model_rebuild()
CreateArtifactCollectionTagAssignmentsInput.model_rebuild()
DeleteArtifactCollectionTagAssignmentsInput.model_rebuild()



from __future__ import annotations

import os
from typing import Any

from wandb import env
from wandb.sdk import wandb_setup
from wandb.sdk.lib import auth as wbauth


def sagemaker_auth(
    overrides: dict[str, Any] | None = None,
    path: str = ".",
    api_key: str | None = None,
) -> None:
    """Write a secrets.env file with the W&B ApiKey and any additional secrets passed.

    Args:
        overrides: Additional environment variables to write to secrets.env
        path: The path to write the secrets file.
    """
    overrides = overrides or dict()

    api_key = (
        overrides.get(env.API_KEY, None)
        or api_key
        or wandb_setup.singleton().settings.api_key
        or wbauth.read_netrc_auth(host=wandb_setup.singleton().settings.base_url)
    )

    if api_key is None:
        raise ValueError(
            "Can't find W&B API key, set the WANDB_API_KEY env variable"
            + " or run `wandb login`"
        )

    overrides[env.API_KEY] = api_key
    with open(os.path.join(path, "secrets.env"), "w") as file:
        for k, v in overrides.items():
            file.write(f"{k}={v}\n")
